/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "AutoConfigurator.hpp"
#include "vfs/VfsLocal.hpp"
#include "FilesEngine.hpp"
#include "XmlReaderWriter.hpp"
#include "Settings.hpp"
#include "Exceptions.hpp"
#include "binreloc.h"

#include <fstream>
using namespace std;
#include<boost/tokenizer.hpp>
using namespace boost;

AutoConfigurator::AutoConfigurator()
: userDirectory(".nao"),
  firstRun(false)
{
	 string pathCfg=br_find_etc_dir(PATH_CFG);

	 if(pathCfg=="/usr/etc")
	 pathCfg="/etc";
	 
	 globalConfigurationFile=pathCfg+string("/nao.conf");
 	 mimeToProgramFile=pathCfg+string("/mime2program.conf");
	

}


AutoConfigurator::~AutoConfigurator()
{
}



bool AutoConfigurator::isFirstRun( )
{
	//if(firstRun==true)
	//return true;
	
	Settings &settings=singleton<Settings>::instance();

	path p=FilesEngine::expandPath( "~" )/path(userDirectory);
	if(!FilesEngine::exists( p ))
	{
		return true;
	}
	else
	{
		if(!settings.isVersionMismatch())
		{
			return false;
		}
		else
		{
			throw ExceptionVersionMismatch();
		}
	}
	return firstRun;
	
}



void AutoConfigurator::firstRunConfiguration( )
{

	
	path userPath=FilesEngine::expandPath( "~" )/path(userDirectory);
	try
	{
		FilesEngine::createDir( userPath );
		path p=userPath/path("python_plugins");
		FilesEngine::createDir( p );
	}
	catch(...){}
	//path src=path(globalConfigurationFile);
	//FilesEngine::copy(src,userPath);
	
}


void AutoConfigurator::clearOldConfiguration()
{
	/*TODO*/
}



void AutoConfigurator::assignProgramsToMime( map<string,vector<Program> > &mimeProgramsMap)
{
	VfsLocal vfs;
	
	try
	{

		path dir="/usr/share/applications";
		Directory directory=vfs.openDir( dir );
		
		while(1)
		{		
			
			File file=directory.getNextFile();
			string name=file.getPath().string();
			ifstream inFile(name.c_str(),ios::in);
			string line;
			Program program;
		
			while( std::getline( inFile, line ) )
			{
		
				if(line.size()>5 && line.substr(0,5)=="Exec=")
				{
					string exec=line.substr(5,line.size()-5);
					if(exec.find(" ")==string::npos)
					exec=exec+" %f";
					
					
					program.setCmd(exec);
					
				}
				else if(line.size()>5 && line.substr(0,5)=="Name=")
				{
					program.setName(line.substr(5,line.size()-5));
				}	
								
				if(program.filled())
				break;				
			}
		
			if(!program.filled())
			continue;
						
			
			
			while( std::getline( inFile, line ) )
			{
				if(line.size()>9 && line.substr(0,9)=="MimeType=")
				{
		
					typedef boost::tokenizer<boost::char_separator<char> >  tokenizer;
  					boost::char_separator<char> sep(";");
  					string line2=line.substr(9,line.size()-9);
  					tokenizer tokens(line2, sep);
	
					for (tokenizer::iterator tokIter = tokens.begin();tokIter != tokens.end(); ++tokIter)
					{						
						vector<Program>vec=mimeProgramsMap[*tokIter];
						vec.push_back(program);	
						mimeProgramsMap[*tokIter]=vec;			
					}
				break;				
				}				
			}

			
			
			
		}
	
	}
	catch(exception e)
	{
				
	}

	XmlReaderWriter xml(false);
	
	try
	{
		xml.loadXml(mimeToProgramFile);

		map<string,Program>programs;

		
		if (xml.openPath("/mime2program/programs/program"))
		{
				
			map<string,string>readMap;
			while ( xml.readNode( readMap ) )
			{
				string name=readMap["name"];
				string command=readMap["command"];
				
				string commandName=command.substr(0,command.find(" "));				
				
				if(FilesEngine::programExist(commandName))
				programs[name]=Program(name,command);	
							
							
				readMap.clear();
				
			}
				
		}



		if (xml.openPath("/mime2program/mimes/mime"))
		{	
			map<string,vector<string> > node;
			
			while ( xml.readNodeVector( node ) )
			{			
				string mimeName=node["name"].at(0);
				vector<string>::iterator iter;	
		
				vector<Program>vec=mimeProgramsMap[mimeName];						
						
				for(iter=node["program"].begin();iter!=node["program"].end();++iter)
				{
					if(programs.find(*iter)!=programs.end())
					vec.push_back(programs[*iter]);
				}
				
				mimeProgramsMap[mimeName]=vec;
			
				node.clear();
			}
		}		
		
	}
	catch(...)
	{
	
	}

}

void AutoConfigurator::setDefaultPrograms( map< string, Program >  mimePrograms,map <string, vector<Program> > mimeProgramsMap )
{

	Settings &settings=singleton<Settings>::instance();

	map< string, Program >::iterator iter;
	for( iter=mimePrograms.begin();iter!=mimePrograms.end() ;++iter )
	{
			string commmandName=iter->second.getName();
			string commmandExec=iter->second.getCmd();
			
			MimeInfo mime=settings.getMime( iter->first );
			mime.setDefaultCommand( commmandName );
			
			vector<Program> progs=mimeProgramsMap[iter->first];
			vector<Program>::iterator iter2;	
			vector<string>additionalCommands;		
			for(iter2=progs.begin();iter2!=progs.end();++iter2)
			{
				if(iter2->getName()!=commmandName)
				additionalCommands.push_back(iter2->getName());			
			}
			mime.setCommands(additionalCommands);
			
			CommandInfo commandInfo(commmandName,"external");
			commandInfo.setExec( commmandExec );
			settings.setCommand(commandInfo);	
			mime.setEmpty(false);		
			settings.setMime( mime );
					
	}


			


}
