/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILESENGINE_HPP
#define FILESENGINE_HPP

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/


#include <boost/filesystem/path.hpp>
#include "File.hpp"
#include "Directory.hpp"
#include "Status.hpp"
#include "Question.hpp"
#include "Vfs.hpp"


#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

using namespace boost::filesystem;

#include <map>
using namespace std;


// Maximum path length
#ifndef MAXPATHLEN
#if defined(PATH_MAX)
#define MAXPATHLEN   PATH_MAX
#elif defined(_MAX_PATH)
#define MAXPATHLEN   _MAX_PATH
#elif defined(MAX_PATH)
#define MAXPATHLEN   MAX_PATH
#else
#define MAXPATHLEN   1024
#endif
#endif


// To search visited inodes
struct inodelist
{
    ino_t st_ino;
    inodelist *next;
};



class FilesEngine
{
public:


	static path getHomeDir();
	static string getCurrentUser();
	static bool programExist(string &programName);
	static vector<string> getCurrentGroups();
	static void runFile(path file);
	static void copy(path &source, path &destination);
	static void createDir(path &dir);
	static void remove(path &p);
	static bool exists(path &p); 
	static string expandPath(const string& path);
	static string printSize(double size);
	static map<string,string> parseParam(string param);
	static string convertTimeToString(time_t time);
    	static string permissionsToString(mode_t mode);
	static void translateErrnoToException(string fileName, string operation);
	static string getEnv(string name);
	static mode_t stringToPremissions(string perm);
	static string permissionsToOctetString(mode_t mode);
	static unsigned long getFreeDiskSize(path p);



};

#endif
