/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Question.hpp"
#include "WindowContent.hpp"
#include "Exceptions.hpp"

#include <map>
#include <string>
#include <iostream>
using namespace std;



Question::Question(QuestionManager * _questionManager):questionManager(_questionManager)
{

}

map< string, string > Question::askCustom(QuestionManager * questionManager, WindowContent & windowContent)
{
	map<string,string> answer;
	questionManager->ask(&windowContent,answer);
	return answer;
}

map< string, string > Question::askCustomQ(WindowContent &windowContent)
{
	return Question::askCustom(questionManager,windowContent);
}


string Question::askString(QuestionManager * questionManager, string question,string text)
{
	WindowContent windowContent;
	windowContent.addWidgetVerticalFrame("","vframe1");
	windowContent.addWidgetLabel("vframe1","label1",question);
	windowContent.addWidgetTextField("vframe1","textField",text,30);
	windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
	windowContent.addWidgetButton("hframe1","ok","ok","ok");
	windowContent.addWidgetButton("hframe1","cancel","cancel","cancel");
	windowContent.setDefaultButton("ok");
	
	map<string,string> answer;
	questionManager->ask(&windowContent,answer);
	

	if(answer["pressedButton"]=="cancel")
	throw ExceptionCancel();

	return answer["textField"];

}
string Question::askStringQ(string question, string text)
{
	return Question::askString(questionManager,question,text);
}


void Question::displayError(QuestionManager * questionManager, string message)
{
	WindowContent windowContent;
	windowContent.addWidgetVerticalFrame("","vframe1");
	windowContent.addWidgetLabel("vframe1","label1",message,"messagebox_critical");
	windowContent.addWidgetButton("vframe1","buttonOk","ok","ok");
	
	map<string,string> answer;
	questionManager->ask(&windowContent,answer);
}

void Question::displayErrorQ(string message)
{
	Question::displayError(questionManager,message);
}

string Question::askOverwrite(QuestionManager * questionManager, File file, string question)
{
	WindowContent windowContent;
	windowContent.addWidgetVerticalFrame("","vframe1");
	/*TODO display file icon, size, date etc*/
	windowContent.addWidgetLabel("vframe1","label1",question+" "+file.getName(),"messagebox_warning");
	windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
	windowContent.addWidgetButton("hframe1","yes","yes","ok");
	windowContent.addWidgetButton("hframe1","all","all");
	windowContent.addWidgetButton("hframe1","skip","skip");
	windowContent.addWidgetButton("hframe1","skip_all","skip all");
	windowContent.addWidgetButton("hframe1","cancel","cancel","cancel");
	
	map<string,string> answer;
	questionManager->ask(&windowContent,answer);
	return answer["pressedButton"];

	
}

string Question::askYesNo(QuestionManager *questionManager,string question)
{
	WindowContent windowContent;
	windowContent.addWidgetVerticalFrame("","vframe1");
	windowContent.addWidgetLabel("vframe1","label1",question);
	windowContent.addWidgetHorizontalFrame("vframe1","hframe1");
	windowContent.addWidgetButton("hframe1","yes","yes","ok");
	windowContent.addWidgetButton("hframe1","no","no","cancel");
	
	map<string,string> answer;
	questionManager->ask(&windowContent,answer);
	return answer["pressedButton"];
	
}

string Question::askYesNoQ(string question)
{
	return Question::askYesNo(questionManager,question);
}

void Question::displayException(QuestionManager * questionManager, ExceptionFileSystem &excep)
{

	string errorMessage=excep.getDescription();
	

	WindowContent windowContent;
	windowContent.addWidgetVerticalFrame("","vframe1");
	windowContent.addWidgetLabel("vframe1","label1",errorMessage,"messagebox_warning");
	windowContent.addWidgetButton("vframe1","cancel","cancel","cancel");
	map<string,string> answer;
	questionManager->ask(&windowContent,answer);
}









