/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SETTINGS_HPP
#define SETTINGS_HPP

#include "XmlReaderWriter.hpp"
#include "CommandInfo.hpp"
#include "MimeInfo.hpp"

#include <boost/filesystem/path.hpp>
#include <vector>
#include <map>
#include <set>
#include <list>
#include "SettingsObservator.hpp"
#include <boost/thread/detail/singleton.hpp>

using namespace boost::filesystem;
using namespace boost::detail::thread;


/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/


class Settings
{
  
public:

     
	path getIconsDir();
	vector<string> getToolbarCommands(string dock);
	void setToolbarCommands(string toolbar, vector<string>commands);
	string getCommandIcon(string command);
	CommandInfo getCommand(string name);
	CommandInfo getCommand(int nr);
	vector<string> getToolbars();
	void	setCommand(CommandInfo cmd);
	vector<string> getAllCommands();
	vector<string> getHeaders(string vfs);
	vector<string> getAvailableHeaders(string vfs);
	void setHeaders(string vfs,vector<string> headers);
	vector<string> getAllVfs();
	vector<int> getHeadersWidth(string vfs);
	void setHeadersWidth(string vfs,vector<int>);
	MimeInfo getMime(string name);
	string getMimeIcon(string name);
	void setMime(MimeInfo mime);
	vector<string> getShutters(void);
	void setShutter(string name,vector<string>commands);
	vector<string> getShutter(string name);
	path getXmlGuiPath(string xmlGuiFile);
	string getFileListStyle(string vfs);
	void setFileListStyle(string vfs, string style );
	string getSortHeader(string vfs);
	string getFontDescription(string fontName);
	void setFontDescription( string fontName, string description );
	vector< string > getTextFieldLast( string command );
	vector< string > getTextFieldCommands();
	string getVfsType(string name);
	void removeCommand(string name);
	void removeMime(string name);
	vector<string> getAllMimeNames();
	void removeShutter(string name);
	
	void addSettingsObservator(SettingsObservator *observator);
	void removeSettingsObservator( SettingsObservator * observator );
	void notifyObservators(void);
	
	void addFilesHistory(string filePath);
	list<string> getFilesHistory();

	void setWindowWidth(const int& theValue)
	{
	    windowWidth = theValue;
	    notifyObservators();
	}
	

	int getWindowWidth() const
	{
	    return windowWidth;
	}

	void setWindowHeight(const int& theValue)
	{
	    windowHeight = theValue;
	    notifyObservators();
	}
	

	int getWindowHeight() const
	{
	    return windowHeight;
	}

	void setDefaultPath(const string& theValue)
	{
	    defaultPath = theValue;
	}
	

	string getDefaultPath() const
	{
	    return defaultPath;
	}

	void setDefaultVfsName(const string& theValue)
	{
	    defaultVfsName = theValue;
	}
	

	string getDefaultVfsName() const
	{
	    return defaultVfsName;
	}

	void setSinglePanel(bool theValue)
	{
	    singlePanel = theValue;
	}
	

	bool isSinglePanel() const
	{
	    return singlePanel;
	}

	void setRightPath ( const string& theValue )
	{
		rightPath = theValue;
	}
	

	string getRightPath() const
	{
		return rightPath;
	}

	void setRightVfsName ( const string& theValue )
	{
		rightVfsName = theValue;
	}
	

	string getRightVfsName() const
	{
		return rightVfsName;
	}

	void setLeftPath ( const string& theValue )
	{
		leftPath = theValue;
	}
	

	string getLeftPath() const
	{
		return leftPath;
	}

	void setLeftVfsName ( const string& theValue )
	{
		leftVfsName = theValue;
	}
	

	string getLeftVfsName() const
	{
		return leftVfsName;
	}

	void setPreviewSize ( int theValue )
	{
		previewSize = theValue;
	}
	

	int getPreviewSize() const
	{
		return previewSize;
	}

	void setHistoryRemoteVfsHostnames ( const vector< string >& theValue )
	{
		historyRemoteVfsHostnames = theValue;
	}
	

	vector< string > getHistoryRemoteVfsHostnames() const
	{
		return historyRemoteVfsHostnames;
	}

	set< string >& getAvailableMime()
	{
		return availableMime;
	}

	void setTypingSpeed ( int theValue )
	{
		typingSpeed = theValue;
	}
	

	int getTypingSpeed() const
	{
		return typingSpeed;
	}

	void setClickSpeed ( int theValue )
	{
		clickSpeed = theValue;
	}
	

	int getClickSpeed() const
	{
		return clickSpeed;
	}

	void setScrollSpeed ( int theValue )
	{
		scrollSpeed = theValue;
	}
	

	int getScrollSpeed() const
	{
		return scrollSpeed;
	}

	void setBlinkSpeed ( int theValue )
	{
		blinkSpeed = theValue;
	}
	

	int getBlinkSpeed() const
	{
		return blinkSpeed;
	}

	void setAnimSpeed ( int theValue )
	{
		animSpeed = theValue;
	}
	

	int getAnimSpeed() const
	{
		return animSpeed;
	}

	void setMenuPause ( int theValue )
	{
		menuPause = theValue;
	}
	

	int getMenuPause() const
	{
		return menuPause;
	}

	void setTooltipPause ( int theValue )
	{
		tooltipPause = theValue;
	}
	

	int getTooltipPause() const
	{
		return tooltipPause;
	}

	void setTooltipTime ( int theValue )
	{
		tooltipTime = theValue;
	}
	

	int getTooltipTime() const
	{
		return tooltipTime;
	}

	void setDragDelta ( int theValue )
	{
		dragDelta = theValue;
	}
	

	int getDragDelta() const
	{
		return dragDelta;
	}

	void setWheelLines ( int theValue )
	{
		wheelLines = theValue;
	}
	

	int getWheelLines() const
	{
		return wheelLines;
	}

	void setScrollDelay ( int theValue )
	{
		scrollDelay = theValue;
	}
	

	int getScrollDelay() const
	{
		return scrollDelay;
	}

	void setBaseColor ( const string& theValue )
	{
		baseColor = theValue;
	}
	

	string getBaseColor() const
	{
		return baseColor;
	}

	void setBorderColor ( const string& theValue )
	{
		borderColor = theValue;
	}
	

	string getBorderColor() const
	{
		return borderColor;
	}

	void setBackColor ( const string& theValue )
	{
		backColor = theValue;
	}
	

	string getBackColor() const
	{
		return backColor;
	}

	void setForeColor ( const string& theValue )
	{
		foreColor = theValue;
	}
	

	string getForeColor() const
	{
		return foreColor;
	}

	void setSelbackColor ( const string& theValue )
	{
		selbackColor = theValue;
	}
	

	string getSelbackColor() const
	{
		return selbackColor;
	}

	void setSelforeColor ( const string& theValue )
	{
		selforeColor = theValue;
	}
	

	string getSelforeColor() const
	{
		return selforeColor;
	}

	void setTipbackColor ( const string& theValue )
	{
		tipbackColor = theValue;
	}
	

	string getTipbackColor() const
	{
		return tipbackColor;
	}

	void setTipforeColor ( const string& theValue )
	{
		tipforeColor = theValue;
	}
	

	string getTipforeColor() const
	{
		return tipforeColor;
	}

	void setMenubackColor ( const string& theValue )
	{
		menubackColor = theValue;
	}
	

	string getMenubackColor() const
	{
		return menubackColor;
	}

	void setMenuforeColor ( const string& theValue )
	{
		menuforeColor = theValue;
	}
	

	string getMenuforeColor() const
	{
		return menuforeColor;
	}

	void setHiliteColor ( const string& theValue )
	{
		hiliteColor = theValue;
	}
	

	string getHiliteColor() const
	{
		return hiliteColor;
	}

	void setShadowColor ( const string& theValue )
	{
		shadowColor = theValue;
	}
	

	string getShadowColor() const
	{
		return shadowColor;
	}

	void setMainColor ( const string& theValue )
	{
		mainColor = theValue;
	}
	

	string getMainColor() const
	{
		return mainColor;
	}

	void setMaxThumbSize ( double theValue )
	{
		maxThumbSize = theValue;
	}
	

	double getMaxThumbSize() const
	{
		return maxThumbSize;
	}

	void setStartupPlacement ( const string& theValue )
	{
		startupPlacement = theValue;
	}
	

	string getStartupPlacement() const
	{
		return startupPlacement;
	}
	
	void setIconsTheme ( const string& theValue )
	{
		iconsTheme = theValue;
	}
	

	string getIconsTheme() const
	{
		return iconsTheme;
	}

	vector< string > getAvailableIconsThemes();

	void setDefaultShutter ( const string& theValue )
	{
		defaultShutter = theValue;
	}
	

	string getDefaultShutter() const
	{
		return defaultShutter;
	}

	void setShowHiddenFiles ( bool theValue )
	{
		showHiddenFiles = theValue;
	}
	

	bool getShowHiddenFiles() const
	{
		return showHiddenFiles;
	}

	string getVersion() const
	{
		return version;
	}
	
	
	bool isVersionMismatch()
	{
		return versionMismatch;
	}

	void setFilesHistorySize ( int theValue )
	{
		filesHistorySize = theValue;
	}

	int getFilesHistorySize() const
	{
		return filesHistorySize;
	}

	void setHistoryRemoteVfsUsers ( const vector< string >& theValue )
	{
		historyRemoteVfsUsers = theValue;
	}
	

	vector< string > getHistoryRemoteVfsUsers() const
	{
		return historyRemoteVfsUsers;
	}

	void setCut ( bool theValue )
	{
		cut = theValue;
	}
	

	bool isCut() const
	{
		return cut;
	}

	void setSaveWindowSize ( bool theValue )
	{
		saveWindowSize = theValue;
	}
	

	bool getSaveWindowSize() const
	{
		return saveWindowSize;
	}
	
	
	
	
	

	




     
private:
        
	Settings();
	~Settings();


	bool versionMismatch;
	
	XmlReaderWriter xml;
	bool singlePanel;
	
	
	map<string,CommandInfo>commands;
	map<string,MimeInfo>mime;
	map<int,CommandInfo>commandsByNumber;
	string dataPath;
	string iconsTheme;
	int windowWidth,windowHeight;
	map<string,vector<string> >toolbarCommands;
	/*FIXME REMOVE ETC*/
	//vector<string>allCommands;
	//vector<string>shutters;
	vector< SettingsObservator*>observators;
	
	string defaultPath;
	string defaultVfsName;
	string rightPath;
	string rightVfsName;
	string leftPath;
	string leftVfsName;
	
	string version;
    
	map< string, vector<string> >  shutters;
	map<string,vector<string> > vfsHeaders;
	map<string,vector<string> > vfsAvailableHeaders;
	map<string,vector<int> > vfsHeadersWidth;
	map<string,string>fontDescriptions;
	map<string,vector<string> >textFieldCommands;
	map<string,string>vfsStyles;
	map<string,string>vfsTypes;
	vector<string>historyRemoteVfsHostnames;
	vector<string>historyRemoteVfsUsers;

	set<string>availableMime;
	
	int previewSize;

	friend class singleton<Settings>;
		
	int typingSpeed;
	int clickSpeed;
	int scrollSpeed;
	int scrollDelay;
	int blinkSpeed;
	int animSpeed;
	int menuPause;
	int tooltipPause;
	int tooltipTime;
	int dragDelta;
	int wheelLines;

	string baseColor;
	string borderColor;
	string backColor;
	string foreColor;
	string selbackColor;
	string selforeColor;
	string tipbackColor;
	string tipforeColor;
	string menubackColor;
	string menuforeColor;
	string hiliteColor;
	string shadowColor;
	string mainColor;
	
	
	double maxThumbSize;
	string startupPlacement;
	vector<string>availableIconsThemes;
	string defaultShutter;
	bool showHiddenFiles;
	list<string>filesHistory;
	int filesHistorySize;
	
	bool saveWindowSize;
	
	bool cut;

};

#endif
