/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef STATUS_HPP
#define STATUS_HPP

#include <iostream>
using namespace std;

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class Status
{
public:
  	virtual void increaseCurrentProgress(double value) { currentSize+=value; totalCurrentSize+=value; }
  	virtual void setCurrentFileSize(double value){ currentSize=0; currentFileSize=value;}
  	virtual void setTotalSize(double value){totalSize=value;}
  	virtual void close(void){closed=true;}
  	virtual void setCurrentFileName(string fileName){currentFileName=fileName;}
  	virtual void tick()=0;
	

	bool isClosed() const
	{
		return closed;
	}
	
  	
protected:
  	
  	Status(): startTime(time(NULL)), closed(false),totalCurrentSize(0)
  	{  		
  	}
  	
  	double getTimeElapsed()
  	{
  		return difftime ( time ( NULL ), startTime );
  	}

	unsigned int getTimeEstimated()
	{
		return static_cast<unsigned int>(getTimeElapsed()*totalSize/currentSize);
	}
	double getSpeed()
	{
		return totalSize/getTimeElapsed();
	}
	
	double getCurrentProgress()
	{
		return 100.0 *currentSize/currentFileSize;
	}
	
	double getTotalProgress()
	{
		return 100.0 *totalCurrentSize/totalSize;
	}
	
	
	bool closed;
	double currentSize;
	double totalCurrentSize;
	double currentFileSize;
	double totalSize;
	string currentFileName;

private:

	time_t startTime;
	

};

#endif
