/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VFSSYNC_HPP
#define VFSSYNC_HPP

#include "Vfs.hpp"

#include <boost/thread/recursive_mutex.hpp>
#include <boost/thread/mutex.hpp>



/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class VfsSync : public Vfs
{
public:
	VfsSync(Vfs* vfs);
	Directory openDir( path &p, Cancelable *cancelable=NULL  );
	
	File readFile(path file);
	void createDir(path p);
	double getTotalSize(path &p, Cancelable *cancelable);
	void copyToFile(File &source, path &destination, Status * status, Cancelable *cancelable);
	void copyFromFile(File &source, path &destination, Status * status, Cancelable *cancelable);
	void copy(Vfs * vfs2,path source, path destination,Status * status,Cancelable *cancelable);
	string getType();
	path symlink(const path& file);
	void init(Panel *panel,QuestionManager *questionManager);
	void chmod(path &source,unsigned int mode);
	void chmodRecursive(path &source,unsigned int mode);
	void removeFile(path &source);
    	void removeDirectory(path &source);
    	void remove(path &source);
    	void rename(path &source, path &destination);
    	File readFullFile(File file);
    	void changeOwnerRecursive(path &source,string user);
    	void changeOwner(path &source,string user);
    	vector<string> getUsers();
    	vector<string> getGroups();
    	void changeGroupRecursive(path &source,string group);
    	void changeGroup(path &source,string group);
    	void createSymlink(path &source,path &destination);
    	
    	
private:

	void closeDir( Directory *directory);
	File readDir( Directory *directory );

Vfs *vfs;
boost::mutex muteX; 
};

#endif
