/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "CmdCopy.hpp"
#include "FilesEngine.hpp"
#include "Exceptions.hpp"

#include <iostream>
using namespace std;


REGISTER_FACTORABLE(CmdCopy);



void CmdCopy::execute( )
{
	

	if(fileList->getVfsType()!="Local" && fileListOpposite->getVfsType()!="Local")
	{
	/*TODO*/
	return;
	}
	bool move=false;
	
	if(getParam()=="move")
	move=true;

	vector<path> files=fileList->getSelectedFileIconsPaths( );
	
	path destinationDirectory;
	
	
	if(fileList->isDragAndDrop())
	{
		destinationDirectory=fileList->getCurrentDirectory();
		fileList->setDragAndDrop(false);
		copy(fileListOpposite,fileList,files,destinationDirectory,move);

	}
	else
	{
		destinationDirectory=fileList->getOpposite()->getCurrentDirectory();	
		copy(fileList,fileListOpposite,files,destinationDirectory,move);

	}

	
	
}

void CmdCopy::init( FileList * _filelist, string _name, CommandInfo _cmdInfo)
{
	CommandOpposite::init(_filelist,_name,_cmdInfo);
	status=fileList->getFileListIcons()->getStatusPanel(this);	
	
}


void CmdCopy::setTotalSize(Vfs *vfs,vector< path > &files, Status * status)
{
	vector<path>::iterator iter;
	
	if(status!=NULL)
	{
		double totalSize=0;
		for(iter=files.begin();iter!=files.end();++iter)
		{
			totalSize+=vfs->getTotalSize(*iter,NULL);
		}
		status->setTotalSize(totalSize);
	} 
}

void CmdCopy::copy(FileList *fileList1, FileList *fileList2, vector< path > files, path destination, bool move)
{
		
	try
	{
		Cancelable *cancelable=this;
		
		vector<path>::iterator iter;
		bool totalSizeCounted=false;
		
		
		if(!move)
		{
		setTotalSize(fileList->getVfs(),files,status);
		totalSizeCounted=true;
		}
		
		bool skip_all=false;
		bool all=false;
		
		for(iter=files.begin();iter!=files.end();++iter)
		{
			path src=*iter;
			path dst=destination/iter->leaf();
			
			if(src==dst)
			continue;

			
			try
			{
				File file=fileList2->getVfs()->readFile(dst);
				/*ask if overwrite*/
				
				if(skip_all)
				continue;
							
				if(all)
				{
					path p=file.getPath();
					fileList2->getVfs()->remove(p);					
				}
				else
				{							
					string answer=Question::askOverwrite(fileList1->getFileListIcons(),file,"overwrite:");
					
					if(answer=="cancel")
					{
						break;
					}
					else if(answer=="skip")
					{
						continue;
					}
					else if(answer=="skip_all")
					{
						skip_all=true;
						continue;
					}
					else if(answer=="all")
					{
						all=true;
						path p=file.getPath();
						fileList2->getVfs()->remove(p);	
						fileList2->removeFileIcon(p.string());
					}
				}
			
			}
			catch(ExceptionReadFile &e){}
			catch(...){}
			
	
		
			if(!move)/*copy*/
			{
				fileList1->getVfs()->copy(fileList2->getVfs(),src,dst,status,cancelable);
				fileList2->addFile(dst.string());
			}
			else /*move*/
			{
				try
				{
					fileList1->getVfs()->rename(src,dst);
					try
					{
					fileList2->addFile(dst.string());
					fileList1->removeFileIcon(src.string());
					}
					catch(ExceptionFileIconNotFound &e){}
				}
				catch(exception &e)
				{
					if(!totalSizeCounted)
					setTotalSize(fileList1->getVfs(),files,status);
			
					fileList1->getVfs()->copy(fileList2->getVfs(),src,dst,status,cancelable);
					
					try
					{
					fileList2->addFile(src.string());
					}
					catch(ExceptionFileIconNotFound &e){}
										
					fileList1->getVfs()->remove(src);
						
					try
					{
					fileList1->removeFileIcon(src.string());
					}
					catch(ExceptionFileIconNotFound &e){}	
								
		
					
				}
			}
		}
	//fileList2->sort();
	}
	catch(ExceptionFileSystem &excep	)
	{
		Question::displayException(fileList->getFileListIcons(),excep);		
	}
	catch(...)
	{
		if(status)
		status->close();
		throw;
	}
	
	if(status)
	status->close();
	
}


