/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FoxConfigurator.hpp"
#include "FoxAutoConfigurator.hpp"

#include <set>
using namespace std;

FXDEFMAP ( FoxConfigurator ) FoxConfiguratorMap[] =
    {
        //________Message_Type_____________________ID____________Message_Handler_______
        FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_MAINWINDOWHEIGHT,FoxConfigurator::ID_MAINWINDOWWIDTH, FoxConfigurator::onChange),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_ACTIVE_COMMAND_CHANGE, FoxConfigurator::onActiveCommandChange),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_ACTIVE_FILETYPE_CHANGE, FoxConfigurator::onActiveFileTypeChange),        
        FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_NEW_COMMAND,FoxConfigurator::ID_NEW_COMMAND_PYTHON_PLUGIN, FoxConfigurator::onNewCommand),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_REMOVE_COMMAND, FoxConfigurator::onRemoveCommand),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_ADD_FILETYPE, FoxConfigurator::onAddFileType),   
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_REMOVE_FILETYPE,FoxConfigurator::onRemoveFileType),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_FILETYPE_WIZARD,FoxConfigurator::onFileTypeWizard),
        FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_UP_COMMAND,FoxConfigurator::ID_DOWN_COMMAND,FoxConfigurator::onUpDownAdditionalCommand),           
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_ADD_COMMAND_ADDITIONAL,FoxConfigurator::onAddAdditionalCommand),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_DEL_COMMAND_ADDITIONAL,FoxConfigurator::onDelAdditionalCommand),
        FXMAPFUNCS ( SEL_COMMAND, FoxConfigurator::ID_CHOOSE_COLOR, FoxConfigurator::ID_CHOOSE_BACKCOLOR, FoxConfigurator::onChooseColor ),
    	FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_REMOVE_COLOR,FoxConfigurator::onRemoveColor),
    	FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_REMOVE_BACK_COLOR,FoxConfigurator::onRemoveBackColor),
    	FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_COLORS,FoxConfigurator::onColorChanged),
    	FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_CHOOSE_FONT1,FoxConfigurator::ID_CHOOSE_FONT3,FoxConfigurator::onChooseFont),
    	FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_VFS_CHANGE,FoxConfigurator::onActiveVfsChange),
    	FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_UP_VFS_HEADER,FoxConfigurator::ID_DOWN_VFS_HEADER,FoxConfigurator::onUpDownVfsHeader),           
        FXMAPFUNCS ( SEL_COMMAND, FoxConfigurator::ID_ADD_HEADER, FoxConfigurator::ID_DEL_HEADER, FoxConfigurator::onAddDelHeader ),
    	FXMAPFUNCS ( SEL_COMMAND, FoxConfigurator::ID_UP_BUTTON_COMMAND, FoxConfigurator::ID_DOWN_SHUTTER_COMMAND, FoxConfigurator::onUpDownShuttersAndToolbars ),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_SHUTTER_CHANGE,FoxConfigurator::onShutterChange),
   	FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_TOOLBAR_CHANGE,FoxConfigurator::onToolbarChange),
        FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_ADD_SHUTTER_COMMAND,FoxConfigurator::ID_DEL_SHUTTER_COMMAND,FoxConfigurator::onAddRemoveShutterCommand),
        FXMAPFUNCS (SEL_COMMAND, FoxConfigurator::ID_ADD_TOOLBAR_COMMAND,FoxConfigurator::ID_DEL_TOOLBAR_COMMAND,FoxConfigurator::onAddRemoveToolbarCommand),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_NEW_SHUTTER,FoxConfigurator::onNewShutter),
        FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_REMOVE_SHUTTER,FoxConfigurator::onRemoveShutter),
     	FXMAPFUNC (SEL_COMMAND, FoxConfigurator::ID_SET_DEFAULT_SHUTTER,FoxConfigurator::onSetDefaultShutter),
     	FXMAPFUNC(SEL_CLOSE,0,FoxConfigurator::onCmdClose),
     	
    };


FXIMPLEMENT ( FoxConfigurator, FXDialogBox, FoxConfiguratorMap, ARRAYNUMBER ( FoxConfiguratorMap ) );





FoxConfigurator::FoxConfigurator ( FXWindow * owner )
: FXDialogBox ( owner, "Configurator", DECOR_ALL, 0, 0, 0, 0, 0, 0, 0, 0, 4, 4 ),
  settings(singleton<Settings>::instance()),
  foxIconManager(FoxSingleton<FoxIconManager>::instance( owner->getApp())),
  app(owner->getApp())
{

	colordlg = new FXColorDialog( this, "Color Dialog" );

	FXVerticalFrame *vertical = new FXVerticalFrame ( this, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y );
	FXHorizontalFrame *horizontal = new FXHorizontalFrame ( vertical, LAYOUT_FILL_X | LAYOUT_FILL_Y );
	buttons = new FXVerticalFrame ( horizontal, LAYOUT_LEFT | LAYOUT_FILL_Y | FRAME_SUNKEN | PACK_UNIFORM_WIDTH | LAYOUT_FIX_HEIGHT, 0, 0, 0, 500, 0, 0, 0, 0, 0, 0 );
	switcher = new FXSwitcher ( horizontal, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0 );
	tooltip = new FXToolTip( getApp() );
	
	icons=foxIconManager.getAllIconsName();
	commands=settings.getAllCommands();

	setupMainSettings();
	setupCommands();
	setupFileType();	
	setupSpeedAndDelays();
	setupLookAndFeel();
	setupVfs();
	setupMenuAndButtons();
	
	new FXSeparator( vertical );
	FXHorizontalFrame *horizontal2 = new FXHorizontalFrame ( vertical, LAYOUT_FILL_X | LAYOUT_FILL_Y );
	
	new FXLabel ( horizontal2, "All changes will be saved after closing configuration window, some changes need program to be restarted. ", foxIconManager.get("about"), LAYOUT_LEFT|ICON_BEFORE_TEXT );
	new FXButton ( horizontal2, "close", foxIconManager.get("ok"), this, ID_CLOSE, FRAME_RAISED | ICON_BEFORE_TEXT | LAYOUT_FILL_Y );
	
	

}





FoxConfigurator::~FoxConfigurator( void )
{

	
}

long FoxConfigurator::onChange (FXObject * sender, FXSelector sel, void *)
{
	if(FXSELID(sel)==ID_MAINWINDOWHEIGHT)
	settings.setWindowHeight(mainwindow_height->getValue());
	else if(FXSELID(sel)==ID_MAINWINDOWWIDTH)
	settings.setWindowWidth(mainwindow_width->getValue());

}

long FoxConfigurator::onActiveCommandChange( FXObject * sender, FXSelector sel , void * )
{
	saveActiveCommand();
}



void FoxConfigurator::setActiveCommand( string activeCommand )
{
	CommandInfo command=settings.getCommand( activeCommand );
	commandsCombo->setCurrentItem(commandsCombo->findItem(activeCommand.c_str()));
	iconsList->setCurrentItem( iconNames[command.getIconName()]);
	shortcutKey->setText(command.getKey().c_str());
	commandType->setText(command.getType().c_str());
	
	commandDescription->setText(command.getDescription().c_str());
	commandGesture->setText(command.getGesture().c_str());
	
	
	if(command.getType()=="external" || command.getType()=="python plugin")
	{
		commandExecute->enable();
		commandExecute->setText(command.getExec().c_str());
	}
	else
	{
		commandExecute->setText("");
		commandExecute->disable();
	}
	
	if(command.getType()=="builtin")
		removeCommandButton->disable();
	else
		removeCommandButton->enable();
		
	if(command.getType()=="external")
	{	
		checkForceReload->enable();
		
		if(command.getForceReload())
			checkForceReload->setCheck(true);	
		else
			checkForceReload->setCheck(false);
	}
	else
	{
		checkForceReload->disable();
	}
	

	if(command.getNeedSelectedFile())
	{	
		checkNeedSelectedFile->setCheck(true);
	}
	else
	{
		checkNeedSelectedFile->setCheck(false);
	}

	
	activeCommandName=command.getName();
	
}

void FoxConfigurator::saveActiveCommand()
{
	CommandInfo command=settings.getCommand(activeCommandName);
	
	if(iconsList->getCurrentItem()!=-1)
	command.setIconName( iconNamesVector[iconsList->getCurrentItem()]);
	
	command.setKey(shortcutKey->getText().text());
	
	if(command.getType()=="external" || command.getType()=="python plugin")
	command.setExec(commandExecute->getText().text());
	
	command.setDescription(commandDescription->getText().text());
	command.setGesture(commandGesture->getText().text());
	
	
	if(checkForceReload->getCheck())
	command.setForceReload(true);
	else
	command.setForceReload(false);
	
	if(checkNeedSelectedFile->getCheck())
	command.setNeedSelectedFile(true);
	else
	command.setNeedSelectedFile(false);
	
	
	
	settings.setCommand(command);

	string newActiveCommandName=commandsCombo->getItem ( commandsCombo->getCurrentItem () ).text ();
	setActiveCommand(newActiveCommandName);
}

long FoxConfigurator::onNewCommand(FXObject * sender, FXSelector sel, void *)
{
	if(newCommandName->getText()!="")
	{
		CommandInfo cmd;
		cmd.setName(newCommandName->getText().text());
		
		if(FXSELID(sel)==ID_NEW_COMMAND)		
		cmd.setType("external");
		else
		cmd.setType("python plugin");
		
		cmd.setForceReload(false);
		
		
		
		settings.setCommand(cmd);
		
		commandsCombo->appendItem( cmd.getName().c_str ());
		defaultCommandList->appendItem( cmd.getName().c_str ());
		additionalCommandsAvailableForButtons->appendItem( cmd.getName().c_str ());
		additionalCommandsAvailable->appendItem( cmd.getName().c_str ());
		
		saveActiveCommand();
		setActiveCommand(cmd.getName());
    	}
}

long FoxConfigurator::onRemoveCommand(FXObject * sender, FXSelector , void *)
{
	commandsCombo->removeItem ( commandsCombo->getCurrentItem () );
	settings.removeCommand(activeCommandName);	
	setActiveCommand(commandsCombo->getItem ( 0 ).text ());
}

long FoxConfigurator::onActiveFileTypeChange(FXObject * sender, FXSelector , void *)
{
	saveActiveFileType();
}

void FoxConfigurator::setActiveFileType(string newActiveFiletypeName)
{
	MimeInfo mimeInfo=settings.getMime( newActiveFiletypeName );
	fileTypeList->setCurrentItem(fileTypeList->findItem(newActiveFiletypeName.c_str()));
	fileTypeIconsList->setCurrentItem( iconNames[mimeInfo.getIcon()]);
	defaultCommandList->setCurrentItem(defaultCommandList->findItem(mimeInfo.getDefaultCommand().c_str()));	
			
	additionalCommands->clearItems();
	vector<string>additionalCommandNames=mimeInfo.getCommands();
	vector<string>::iterator iter;
	for(iter=additionalCommandNames.begin();iter!=additionalCommandNames.end();++iter)
	{
		additionalCommands->appendItem(iter->c_str(),foxIconManager.get(settings.getCommandIcon( *iter )));
	}
	
	/*TODO*/
	
	if(mimeInfo.getColor()=="")
	{
		setAllStandardColor(colorbutton);	
	}
	else
	{
		setAllColor( colorbutton, fxcolorfromname(mimeInfo.getColor().c_str()) );
	}
	
	if(mimeInfo.getBackColor()=="")
	{
		setAllStandardColor(backcolorbutton);
	}
	else
	{
        	setAllColor( backcolorbutton, fxcolorfromname(mimeInfo.getBackColor().c_str()) );
        }
	
		
	activeFileTypeName=mimeInfo.getName();
}

void FoxConfigurator::saveActiveFileType()
{

	MimeInfo mimeInfo=settings.getMime(activeFileTypeName);
	
	if(fileTypeIconsList->getCurrentItem()!=-1)
	mimeInfo.setIcon( iconNamesVector[fileTypeIconsList->getCurrentItem()]);
	
	if(defaultCommandList->getCurrentItem()!=-1)
	mimeInfo.setDefaultCommand(defaultCommandList->getItem(defaultCommandList->getCurrentItem()).text());
			
	vector<string>commandsVec;
	
	for ( int i = 0; i < additionalCommands->getNumItems (); i++ )
	{
		commandsVec.push_back( additionalCommands->getItemText( i ).text() );
	}
	
	FXchar buffer[ 60 ];
	
	if(colorbutton->getBaseColor()==colorbutton->getBorderColor())
	{
		mimeInfo.setColor(fxnamefromcolor( buffer, colorbutton->getBaseColor() ));
	}
	else
	{
		mimeInfo.setColor("");
	}
	
	if(backcolorbutton->getBaseColor()==backcolorbutton->getBorderColor())
	{
		mimeInfo.setBackColor(fxnamefromcolor( buffer, backcolorbutton->getBaseColor() ));
	}
	else
	{
		mimeInfo.setBackColor("");
	}
	
	mimeInfo.setCommands(commandsVec);
	
	
	
	settings.setMime(mimeInfo);

	string newActiveFiletypeName=fileTypeList->getItem ( fileTypeList->getCurrentItem () ).text ();
	setActiveFileType(newActiveFiletypeName);
}

long FoxConfigurator::onAddFileType(FXObject * sender, FXSelector , void *)
{
	string newMimeTypeName=availableMimeList->getItem(availableMimeList->getCurrentItem()).text();
	fileTypeList->appendItem(newMimeTypeName.c_str());
	MimeInfo newMime(newMimeTypeName);
	settings.setMime(newMime);
	saveActiveFileType();
	setActiveFileType(newMimeTypeName);
}

long FoxConfigurator::onRemoveFileType(FXObject * sender, FXSelector , void *)
{
	fileTypeList->removeItem ( fileTypeList->getCurrentItem () );
	settings.removeMime(activeFileTypeName);
	setActiveFileType(fileTypeList->getItem ( 0 ).text ());

}

long FoxConfigurator::onFileTypeWizard(FXObject * sender, FXSelector , void *)
{
	FoxAutoConfigurator foxAutoConfigurator(app);
	foxAutoConfigurator.setMimeAssociations();
}



long FoxConfigurator::onUpDownAdditionalCommand(FXObject * sender, FXSelector sel, void *)
{
	if ( additionalCommands->getNumItems () == 0 )
		return 0;
	
	FXushort id = FXSELID( sel );
	if ( id == ID_UP_COMMAND )
	{
		if ( additionalCommands->getCurrentItem() != 0 )
		additionalCommands->moveItem( additionalCommands->getCurrentItem() - 1, additionalCommands->getCurrentItem() );
	}
	else
	{
		if ( additionalCommands->getCurrentItem() != additionalCommands->getNumItems () - 1 )
		additionalCommands->moveItem( additionalCommands->getCurrentItem() + 1, additionalCommands->getCurrentItem() );
	}
	
	saveActiveFileType();
	


}

long FoxConfigurator::onDelAdditionalCommand(FXObject * sender, FXSelector , void *)
{
	additionalCommands->removeItem( additionalCommands->getCurrentItem() );
}

long FoxConfigurator::onAddAdditionalCommand(FXObject * sender, FXSelector , void *)
{
	 additionalCommands->appendItem( additionalCommandsAvailable->getItemText( additionalCommandsAvailable->getCurrentItem() ) );
}

void FoxConfigurator::setAllColor( FXButton* button, FXColor color )
{
    button->setBaseColor( color );
    button->setBorderColor( color );
    button->setShadowColor( color );
    button->setHiliteColor( color );
    button->setTextColor( color );
    button->setBackColor( color );
}

void FoxConfigurator::setAllStandardColor(FXButton * button)
{
    button->setBaseColor( app->getBaseColor() );
    button->setBorderColor( app->getBorderColor() );
    button->setShadowColor( app->getShadowColor() );
    button->setHiliteColor( app->getHiliteColor() );
    button->setTextColor( FXRGB(0,0,0) ); /*TODO*/
    button->setBackColor( app->getBackColor() );


}

long FoxConfigurator::onChooseColor ( FXObject * sender, FXSelector sel, void * )
{

    if ( !colordlg->execute() )
        return 0;


    FXColor color = colordlg->getRGBA();
    FXushort id = FXSELID( sel );
    if ( id == ID_CHOOSE_COLOR )
        setAllColor( colorbutton, color );
    else
        setAllColor( backcolorbutton, color );


}

long FoxConfigurator::onRemoveColor(FXObject * sender, FXSelector sel, void *)
{	
	setAllStandardColor(colorbutton);
}

long FoxConfigurator::onRemoveBackColor(FXObject * sender, FXSelector sel, void *)
{
	setAllStandardColor(backcolorbutton);
}

void FoxConfigurator::saveSpeedAndDelays()
{
	settings.setTypingSpeed(typingSpeed);
	settings.setClickSpeed(clickSpeed);
	settings.setScrollSpeed(scrollSpeed);
	settings.setScrollDelay(scrollDelay);
	settings.setBlinkSpeed(blinkSpeed);
	settings.setAnimSpeed(animSpeed);
	settings.setMenuPause(menuPause);
	settings.setTooltipPause(tooltipPause);
	settings.setTooltipTime(tooltipTime);
	settings.setDragDelta(dragDelta);
	settings.setWheelLines(wheelLines);

}



void FoxConfigurator::setupCommands()
{
	FXVerticalFrame *commandsPane = new FXVerticalFrame ( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
      	new FXButton ( buttons, "Commands Settings", foxIconManager.get("commands_settings"), switcher, FXSwitcher::ID_OPEN_SECOND, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
    	commandsCombo = new FXListBox ( commandsPane, this, ID_ACTIVE_COMMAND_CHANGE );
    	
    	vector<string>::iterator iter;
    		for(iter=commands.begin();iter!=commands.end();++iter)
    		{
    			commandsCombo->appendItem( iter->c_str (),foxIconManager.get(settings.getCommandIcon( *iter )) );    			
    		}
    		commands.size()<20 ? commandsCombo->setNumVisible(commands.size()+1): commandsCombo->setNumVisible( 20 );
	
	new FXLabel ( commandsPane, "icon: ", NULL, LAYOUT_LEFT );
    	iconsList = new FXListBox ( commandsPane );
    	
	
	int pos=0;
	
	iconsList->appendItem( "",NULL);
    	iconNames[""]=pos++;
    	iconNamesVector.push_back("");
	
   		for(iter=icons.begin();iter!=icons.end();++iter)
    		{
    			iconsList->appendItem( "",foxIconManager.get(*iter ));
    			iconNames[*iter]=pos++;
    			iconNamesVector.push_back(*iter);
    		}
    		icons.size()<20 ? iconsList->setNumVisible(icons.size()+1):iconsList->setNumVisible( 20 );
    		
	new FXLabel(commandsPane,"key shorcut (example: ctrl+k, f1, F2):");
	shortcutKey=new FXTextField ( commandsPane,20);
	
	new FXLabel(commandsPane,"command type:");
	commandType=new FXLabel(commandsPane,"");
	
	new FXLabel(commandsPane,"execute:");
	commandExecute=new FXTextField ( commandsPane,20);
	
	new FXLabel(commandsPane,"description:");
	commandDescription=new FXTextField ( commandsPane,20);
	
	new FXLabel(commandsPane,"mouse gesture (see examples on website):");
	commandGesture=new FXTextField ( commandsPane,20);
	
	
	checkForceReload = new FXCheckButton ( commandsPane, "force reload directory", NULL, 0, JUSTIFY_LEFT | JUSTIFY_TOP | ICON_BEFORE_TEXT | LAYOUT_SIDE_TOP );
	checkNeedSelectedFile = new FXCheckButton ( commandsPane, "need selected file", NULL, 0, JUSTIFY_LEFT | JUSTIFY_TOP | ICON_BEFORE_TEXT | LAYOUT_SIDE_TOP );
	
	
	new FXSeparator( commandsPane );
        FXHorizontalFrame *hfr = new FXHorizontalFrame ( commandsPane, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        removeCommandButton = new FXButton ( hfr, "Remove", foxIconManager.get("minus"), this, ID_REMOVE_COMMAND, FRAME_RAISED|ICON_BEFORE_TEXT );
        new FXButton ( hfr, "New Command", foxIconManager.get("plus"), this, ID_NEW_COMMAND, FRAME_RAISED|ICON_BEFORE_TEXT  );
        new FXButton ( hfr, "New Python Command", foxIconManager.get("plus"), this, ID_NEW_COMMAND_PYTHON_PLUGIN, FRAME_RAISED|ICON_BEFORE_TEXT  );
       
        newCommandName = new FXTextField ( hfr, 20 );
	
	
	
	setActiveCommand(*commands.begin());
}


void FoxConfigurator::setupFileType()
{
	FXVerticalFrame *fileTypePane = new FXVerticalFrame ( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
      	new FXButton ( buttons, "File Type Settings", foxIconManager.get("file_types_settings"), switcher, FXSwitcher::ID_OPEN_THIRD, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
    	
	
	fileTypeList = new FXListBox ( fileTypePane, this, ID_ACTIVE_FILETYPE_CHANGE );
    	vector<string> fileTypes=settings.getAllMimeNames();
	vector<string>::iterator iter;
	
    		for(iter=fileTypes.begin();iter!=fileTypes.end();++iter)
    		{
    			fileTypeList->appendItem( iter->c_str (),foxIconManager.get(settings.getMimeIcon(*iter)) );
    		}
    		fileTypes.size()<20 ? fileTypeList->setNumVisible(fileTypes.size()+1): fileTypeList->setNumVisible( 20 );
	
	new FXLabel ( fileTypePane, "icon: ", NULL, LAYOUT_LEFT );
    	fileTypeIconsList = new FXListBox ( fileTypePane );    	

	fileTypeIconsList->appendItem( "",NULL);
		
   		for(iter=icons.begin();iter!=icons.end();++iter)
    		{
    			fileTypeIconsList->appendItem( "",foxIconManager.get(*iter ));
       		}
    		icons.size()<20 ? fileTypeIconsList->setNumVisible(icons.size()+1):fileTypeIconsList->setNumVisible( 20 );
    		
    	new FXLabel(fileTypePane,"default command");	
    	defaultCommandList=new FXListBox ( fileTypePane );

	defaultCommandList->appendItem("");
    		for(iter=commands.begin();iter!=commands.end();++iter)
    		{
    			defaultCommandList->appendItem( iter->c_str (),foxIconManager.get(settings.getCommandIcon( *iter )) );    			
    		}
    		commands.size()<20 ? defaultCommandList->setNumVisible(commands.size()+1): defaultCommandList->setNumVisible( 20 );	
    		
    		
    	FXHorizontalFrame* hframe1 = new FXHorizontalFrame ( fileTypePane, 0 , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        new FXLabel( hframe1, "text color:" );
        colorbutton = new FXButton( hframe1, "set text color", NULL, this, ID_CHOOSE_COLOR );
        	      new FXButton( hframe1, "remove text color", NULL, this, ID_REMOVE_COLOR );
        hframe1 = new FXHorizontalFrame ( fileTypePane, 0 , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        new FXLabel( hframe1, "back color:" );
        backcolorbutton = new FXButton( hframe1, "set back color", NULL, this, ID_CHOOSE_BACKCOLOR );
       			  new FXButton( hframe1, "remove back color", NULL, this, ID_REMOVE_BACK_COLOR );	
    		
    		
    	FXHorizontalFrame* hzframe = new FXHorizontalFrame ( fileTypePane, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        FXVerticalFrame* vframe0 = new FXVerticalFrame ( hzframe, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        new FXLabel( vframe0, "additional commands:" );
        additionalCommands = new FXList ( vframe0, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
        additionalCommands->setNumVisible( 5 );
        FXVerticalFrame* vframe = new FXVerticalFrame ( hzframe, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        new FXLabel( vframe, "\n" );
        new FXArrowButton( vframe, this, ID_UP_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_UP );
        new FXArrowButton( vframe, this, ID_DOWN_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_DOWN );
        new FXArrowButton( vframe, this, ID_ADD_COMMAND_ADDITIONAL, FRAME_RAISED | FRAME_THICK | ARROW_LEFT );
        new FXArrowButton( vframe, this, ID_DEL_COMMAND_ADDITIONAL, FRAME_RAISED | FRAME_THICK | ARROW_RIGHT );
        FXVerticalFrame *vframe1 = new FXVerticalFrame ( hzframe, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
        new FXLabel( vframe1, "all available commands:" );
        additionalCommandsAvailable = new FXList ( vframe1, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
        additionalCommandsAvailable->setNumVisible( 5 );	
    		
    	for(iter=commands.begin();iter!=commands.end();++iter)
    	{
					additionalCommandsAvailable->appendItem( iter->c_str (),foxIconManager.get(settings.getCommandIcon( *iter )) );    			
    	}	
    		
  
	new FXSeparator( fileTypePane );
	
	new FXButton ( fileTypePane, "Remove", foxIconManager.get("minus"), this, ID_REMOVE_FILETYPE, FRAME_RAISED|ICON_BEFORE_TEXT );
	new FXSeparator( fileTypePane );
	
	new FXLabel(fileTypePane,"available file types");	
	availableMimeList=new FXListBox ( fileTypePane );
	set<string>& availableMime=settings.getAvailableMime();
 	set<string>::iterator setIter;
	
	for(setIter=availableMime.begin();setIter!=availableMime.end();++setIter)
	{
		availableMimeList->appendItem(setIter->c_str());
	}
	availableMimeList->setNumVisible(20);
	
	new FXButton ( fileTypePane, "Add", foxIconManager.get("plus"), this, ID_ADD_FILETYPE, FRAME_RAISED|ICON_BEFORE_TEXT  );
	
	new FXSeparator( fileTypePane );
	new FXLabel(fileTypePane,"Will assign programs to all file types.");
	new FXButton ( fileTypePane, "Wizard", foxIconManager.get("about"), this,ID_FILETYPE_WIZARD , FRAME_RAISED|ICON_BEFORE_TEXT );
	
	
	setActiveFileType(*fileTypes.begin());
}


void FoxConfigurator::setupLookAndFeel()
{

	new FXButton ( buttons, "Look and feel",foxIconManager.get("look_and_feel"), switcher, FXSwitcher::ID_OPEN_FIFTH, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
	

	/// Retrieve Current Color Settings
	hilite = getApp() ->getHiliteColor();
	shadow = getApp() ->getShadowColor();
	base = getApp() ->getBaseColor();
	back = getApp() ->getBackColor();
	border = getApp() ->getBorderColor();
	fore = getApp() ->getForeColor();
	selfore = getApp() ->getSelforeColor();
	selback = getApp() ->getSelbackColor();
	tipfore = getApp() ->getTipforeColor();
	tipback = getApp() ->getTipbackColor();
	menufore = getApp() ->getSelMenuTextColor();
	menuback = getApp() ->getSelMenuBackColor();
	/*TODO*/
	//main = fxcolorfromname( conf->readonestring ( "/NaoConfig/colors/main" ).c_str() );
	
	target_base.connect( base );
	target_back.connect( back );
	target_border.connect( border );
	target_fore.connect( fore );
	target_hilite.connect( hilite );
	target_shadow.connect( shadow );
	target_selfore.connect( selfore );
	target_selback.connect( selback );
	target_tipfore.connect( tipfore );
	target_tipback.connect( tipback );
	target_menufore.connect( menufore );
	target_menuback.connect( menuback );
	target_main.connect( main );
	
	target_base.setTarget( this );
	target_back.setTarget( this );
	target_border.setTarget( this );
	target_fore.setTarget( this );
	target_hilite.setTarget( this );
	target_shadow.setTarget( this );
	target_selfore.setTarget( this );
	target_selback.setTarget( this );
	target_tipfore.setTarget( this );
	target_tipback.setTarget( this );
	target_menufore.setTarget( this );
	target_menuback.setTarget( this );
	target_main.setTarget( this );
	
	target_base.setSelector( ID_COLORS );
	target_back.setSelector( ID_COLORS );
	target_border.setSelector( ID_COLORS );
	target_fore.setSelector( ID_COLORS );
	target_hilite.setSelector( ID_COLORS );
	target_shadow.setSelector( ID_COLORS );
	target_selfore.setSelector( ID_COLORS );
	target_selback.setSelector( ID_COLORS );
	target_tipfore.setSelector( ID_COLORS );
	target_tipback.setSelector( ID_COLORS );
	target_menufore.setSelector( ID_COLORS );
	target_menuback.setSelector( ID_COLORS );
	target_main.setSelector( ID_COLORS );

	font[ 0 ] = new FXFont( getApp(), settings.getFontDescription("normal").c_str() );
	font[ 0 ] ->create();
	font[ 1 ] = new FXFont( getApp(), settings.getFontDescription("big").c_str() );
	font[ 1 ] ->create();
	font[ 2 ] = new FXFont( getApp(), settings.getFontDescription("small").c_str() );
	font[ 2 ] ->create();

		
		
	FXVerticalFrame *vframe = new FXVerticalFrame( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	FXHorizontalFrame *hframe = new FXHorizontalFrame( vframe, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	new FXSeparator( vframe, SEPARATOR_GROOVE | LAYOUT_FILL_X );
	
	
	FXVerticalFrame *frame = new FXVerticalFrame( hframe, LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	new FXSeparator( hframe, SEPARATOR_GROOVE | LAYOUT_FILL_Y );
	
	
	
	FXMatrix* matrix = new FXMatrix( frame, 2, LAYOUT_FILL_Y | MATRIX_BY_COLUMNS, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, 1, 1 );
	
	FXColorWell *colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_base, FXDataTarget::ID_VALUE );
	FXLabel *label = new FXLabel( matrix, "Base Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_border, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Border Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_fore, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Text Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_back, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Background Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_selfore, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Selected Text Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_selback, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Selected Background Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_menufore, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Selected Menu Text Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_menuback, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Selected Menu Background Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_tipfore, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Tip Text Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_tipback, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Tip Background Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_shadow, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Shadow Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_hilite, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Hilite Color" );
	
	colorwell = new FXColorWell( matrix, FXRGB( 0, 0, 255 ), &target_main, FXDataTarget::ID_VALUE );
	label = new FXLabel( matrix, "Main Color" );
	
	frame = new FXVerticalFrame( hframe, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, 0, 0 );
	
	tabbook = new FXTabBook( frame, NULL, 0, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0 );
	tabitem = new FXTabItem( tabbook, " Item 1 " );
	tabframe = new FXVerticalFrame( tabbook, LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_THICK | FRAME_RAISED );
	
	labeltextframe1 = new FXHorizontalFrame( tabframe, LAYOUT_FILL_X );
	label1 = new FXLabel( labeltextframe1, "Label with Text", NULL );
	textfield1 = new FXTextField( labeltextframe1, 30, NULL, 0, LAYOUT_FILL_X | FRAME_THICK | FRAME_SUNKEN );
	textfield1->setText( "Select this text, to see the selected colors" );
	
	labeltextframe2 = new FXHorizontalFrame( tabframe, LAYOUT_FILL_X );
	textframe1 = new FXHorizontalFrame( labeltextframe2, LAYOUT_FILL_X | FRAME_THICK | FRAME_SUNKEN, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0 );
	label3 = new FXLabel( textframe1, "Selected Text (with focus)", NULL, LAYOUT_FILL_X, 0, 0, 0, 0, 1, 1, 1, 1 );
	textframe2 = new FXHorizontalFrame( labeltextframe2, LAYOUT_FILL_X | FRAME_THICK | FRAME_SUNKEN, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0 );
	label4 = new FXLabel( textframe2, "Selected Text (no focus)", NULL, LAYOUT_FILL_X, 0, 0, 0, 0, 1, 1, 1, 1 );
	
	sep1 = new FXSeparator( tabframe, LAYOUT_FILL_X | SEPARATOR_LINE );
	
	tabsubframe = new FXHorizontalFrame( tabframe, LAYOUT_FILL_X | LAYOUT_FILL_Y );
	
	grpbox1 = new FXGroupBox( tabsubframe, "MenuPane", FRAME_GROOVE | LAYOUT_FILL_Y | LAYOUT_FILL_X );
	
	menuframe = new FXVerticalFrame( grpbox1, FRAME_RAISED | FRAME_THICK | LAYOUT_CENTER_X | LAYOUT_CENTER_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	menulabels[ 0 ] = new FXLabel( menuframe, "&Open", NULL, LABEL_NORMAL, 0, 0, 0, 0, 16, 4 );
	menulabels[ 1 ] = new FXLabel( menuframe, "S&ave", NULL, LABEL_NORMAL, 0, 0, 0, 0, 16, 4 );
	sep2 = new FXSeparator( menuframe, LAYOUT_FILL_X | SEPARATOR_GROOVE );
	menulabels[ 2 ] = new FXLabel( menuframe, "I&mport", NULL, LABEL_NORMAL, 0, 0, 0, 0, 16, 4 );
	menulabels[ 4 ] = new FXLabel( menuframe, "Selected Menu Entry", NULL, LABEL_NORMAL, 0, 0, 0, 0, 16, 4 );
	menulabels[ 3 ] = new FXLabel( menuframe, "Print", NULL, LABEL_NORMAL, 0, 0, 0, 0, 16, 4 );
	sep3 = new FXSeparator( menuframe, LAYOUT_FILL_X | SEPARATOR_GROOVE );
	menulabels[ 5 ] = new FXLabel( menuframe, "&Quit", NULL, LABEL_NORMAL, 0, 0, 0, 0, 16, 4 );
	
	grpbox2 = new FXGroupBox( tabsubframe, "Tooltips", FRAME_GROOVE | LAYOUT_FILL_Y | LAYOUT_FILL_X );
	
	label2 = new FXLabel( grpbox2, "Sample Tooltip", NULL, FRAME_LINE | LAYOUT_CENTER_X );
	label5 = new FXLabel( grpbox2, "Multiline Sample\n Tooltip", NULL, FRAME_LINE | LAYOUT_CENTER_X );
	
	hframe = new FXHorizontalFrame( vframe, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING );
	
	new FXLabel( hframe, "Normal Font: ", NULL, LAYOUT_CENTER_Y );
	fontbutton[ 0 ] = new FXButton( hframe, " ", NULL, this, ID_CHOOSE_FONT1, LAYOUT_CENTER_Y | FRAME_RAISED | JUSTIFY_CENTER_X | JUSTIFY_CENTER_Y | LAYOUT_FILL_X );
	
	hframe = new FXHorizontalFrame( vframe, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING );
	new FXLabel( hframe, "Big Font: ", NULL, LAYOUT_CENTER_Y );
	fontbutton[ 1 ] = new FXButton( hframe, " ", NULL, this, ID_CHOOSE_FONT2, LAYOUT_CENTER_Y | FRAME_RAISED | JUSTIFY_CENTER_X | JUSTIFY_CENTER_Y | LAYOUT_FILL_X );
	
	hframe = new FXHorizontalFrame( vframe, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING );
	new FXLabel( hframe, "Small Font: ", NULL, LAYOUT_CENTER_Y );
	fontbutton[ 2 ] = new FXButton( hframe, " ", NULL, this, ID_CHOOSE_FONT3, LAYOUT_CENTER_Y | FRAME_RAISED | JUSTIFY_CENTER_X | JUSTIFY_CENTER_Y | LAYOUT_FILL_X );
	
	setupColors();
	setupFont(0);
	setupFont(1);
	setupFont(2);
	
}

void FoxConfigurator::setupSpeedAndDelays()
{
	new FXButton ( buttons, "Speed and delays", foxIconManager.get("speed_and_delays"), switcher, FXSwitcher::ID_OPEN_FOURTH, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
	
	
	typingSpeed = getApp() ->getTypingSpeed();
	clickSpeed = getApp() ->getClickSpeed();
	scrollSpeed = getApp() ->getScrollSpeed();
	scrollDelay = getApp() ->getScrollDelay();
	blinkSpeed = getApp() ->getBlinkSpeed();
	animSpeed = getApp() ->getAnimSpeed();
	menuPause = getApp() ->getMenuPause();
	tooltipPause = getApp() ->getTooltipPause();
	tooltipTime = getApp() ->getTooltipTime();
	dragDelta = getApp() ->getDragDelta();
	wheelLines = getApp() ->getWheelLines();
	
	
	target_typingspeed.connect( typingSpeed );
	target_clickspeed.connect( clickSpeed );
	target_scrollspeed.connect( scrollSpeed );
	target_scrolldelay.connect( scrollDelay );
	target_blinkspeed.connect( blinkSpeed );
	target_animspeed.connect( animSpeed );
	target_menupause.connect( menuPause );
	target_tooltippause.connect( tooltipPause );
	target_tooltiptime.connect( tooltipTime );
	target_dragdelta.connect( dragDelta );
	target_wheellines.connect( wheelLines );
	
	
	FXHorizontalFrame *hframe = new FXHorizontalFrame( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	FXMatrix* matrix = new FXMatrix( hframe, 3, LAYOUT_FILL_Y | MATRIX_BY_COLUMNS, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING );
	
	const FXuint spinnerstyle = FRAME_SUNKEN | FRAME_THICK;
	
	new FXLabel( matrix, "Typing Speed\t\tTyping Speed", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	FXSpinner *spinner = new FXSpinner( matrix, 4, &target_typingspeed, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 500 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	
	new FXLabel( matrix, "Double Click Speed\t\tDouble Click Speed", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_clickspeed, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 100 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Scroll Speed\t\tScroll Speed", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_scrollspeed, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 10 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Scroll Delay\t\tScroll Delay", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_scrolldelay, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 100 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Cursor Blink Speed\t\tCursor Blink Speed", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_blinkspeed, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 100 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Animation Speed\t\tAnimation Speed", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	new FXSpinner( matrix, 4, &target_animspeed, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 1 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Cascade Menu Popup Delay\t\tAmount of delay before cascading menu is shown", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_menupause, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 100 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Tooltip Popup Delay\t\tAmount of Delay before tooltip is shown ", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_tooltippause, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 100 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Tooltip Time\t\tTime that tooltips are shown", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_tooltiptime, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 500 );
	new FXLabel( matrix, "ms", NULL, LAYOUT_CENTER_Y );
	
	new FXLabel( matrix, "Drag Delta\t\tMinimum distance considered a mouse move", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_dragdelta, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 9999 );
	spinner->setIncrement( 1 );
	new FXFrame( matrix, FRAME_NONE );
	
	new FXLabel( matrix, "Wheel Lines", NULL, LAYOUT_RIGHT | LAYOUT_CENTER_Y );
	spinner = new FXSpinner( matrix, 4, &target_wheellines, FXDataTarget::ID_VALUE, spinnerstyle );
	spinner->setRange( 1, 100 );
}

void FoxConfigurator::setupColors()
{
	//shadow = makeShadowColor(base);
	//hilite = makeHiliteColor(base);
	
	tabitem->setBorderColor( border );
	tabitem->setBaseColor( base );
	tabitem->setBackColor( base );
	tabitem->setTextColor( fore );
	tabitem->setShadowColor( shadow );
	tabitem->setHiliteColor( hilite );
	
	tabframe->setBorderColor( border );
	tabframe->setBaseColor( base );
	tabframe->setBackColor( base );
	tabframe->setShadowColor( shadow );
	tabframe->setHiliteColor( hilite );
	
	
	tabsubframe->setBorderColor( border );
	tabsubframe->setBaseColor( base );
	tabsubframe->setBackColor( base );
	tabsubframe->setShadowColor( shadow );
	tabsubframe->setHiliteColor( hilite );
	
	menuframe->setBorderColor( border );
	menuframe->setBaseColor( base );
	menuframe->setBackColor( base );
	menuframe->setShadowColor( shadow );
	menuframe->setHiliteColor( hilite );
	
	grpbox1->setBorderColor( border );
	grpbox1->setBaseColor( base );
	grpbox1->setBackColor( base );
	grpbox1->setShadowColor( shadow );
	grpbox1->setHiliteColor( hilite );
	grpbox1->setTextColor( fore );
	
	grpbox2->setBorderColor( border );
	grpbox2->setBaseColor( base );
	grpbox2->setBackColor( base );
	grpbox2->setShadowColor( shadow );
	grpbox2->setHiliteColor( hilite );
	grpbox2->setTextColor( fore );
	
	sep1->setBorderColor( border );
	sep1->setBaseColor( base );
	sep1->setBackColor( base );
	sep1->setShadowColor( shadow );
	sep1->setHiliteColor( hilite );
	
	sep2->setBorderColor( border );
	sep2->setBaseColor( base );
	sep2->setBackColor( base );
	sep2->setShadowColor( shadow );
	sep2->setHiliteColor( hilite );
	
	sep3->setBorderColor( border );
	sep3->setBaseColor( base );
	sep3->setBackColor( base );
	sep3->setShadowColor( shadow );
	sep3->setHiliteColor( hilite );
	
	labeltextframe1->setBorderColor( border );
	labeltextframe1->setBaseColor( base );
	labeltextframe1->setBackColor( base );
	labeltextframe1->setShadowColor( shadow );
	labeltextframe1->setHiliteColor( hilite );
	
	
	labeltextframe2->setBorderColor( border );
	labeltextframe2->setBaseColor( base );
	labeltextframe2->setBackColor( base );
	labeltextframe2->setShadowColor( shadow );
	labeltextframe2->setHiliteColor( hilite );
	
	label1->setBorderColor( border );
	label1->setBaseColor( base );
	label1->setBackColor( base );
	label1->setTextColor( fore );
	label1->setShadowColor( shadow );
	label1->setHiliteColor( hilite );
	
	label2->setBorderColor( tipfore );
	label2->setBaseColor( tipback );
	label2->setBackColor( tipback );
	label2->setTextColor( tipfore );
	label2->setShadowColor( shadow );
	label2->setHiliteColor( hilite );
	
	label3->setBorderColor( border );
	label3->setBaseColor( base );
	label3->setBackColor( selback );
	label3->setTextColor( selfore );
	label3->setShadowColor( shadow );
	label3->setHiliteColor( hilite );
	
	label4->setBorderColor( border );
	label4->setBaseColor( base );
	label4->setBackColor( base );
	label4->setTextColor( fore );
	label4->setShadowColor( shadow );
	label4->setHiliteColor( hilite );
	
	label5->setBorderColor( tipfore );
	label5->setBaseColor( tipback );
	label5->setBackColor( tipback );
	label5->setTextColor( tipfore );
	label5->setShadowColor( shadow );
	label5->setHiliteColor( hilite );
	
	for ( int i = 0;i < 6;i++ )
	{
		menulabels[ i ] ->setBorderColor( border );
		menulabels[ i ] ->setBaseColor( base );
		menulabels[ i ] ->setBackColor( base );
		menulabels[ i ] ->setTextColor( fore );
		menulabels[ i ] ->setShadowColor( shadow );
		menulabels[ i ] ->setHiliteColor( hilite );
	}
	
	menulabels[ 4 ] ->setBorderColor( border );
	menulabels[ 4 ] ->setBaseColor( menuback );
	menulabels[ 4 ] ->setBackColor( menuback );
	menulabels[ 4 ] ->setTextColor( menufore );
	menulabels[ 4 ] ->setShadowColor( shadow );
	menulabels[ 4 ] ->setHiliteColor( hilite );
	
	textframe1->setBorderColor( border );
	textframe1->setBaseColor( base );
	textframe1->setBackColor( back );
	textframe1->setShadowColor( shadow );
	textframe1->setHiliteColor( hilite );
	
	textframe2->setBorderColor( border );
	textframe2->setBaseColor( base );
	textframe2->setBackColor( back );
	textframe2->setShadowColor( shadow );
	textframe2->setHiliteColor( hilite );
	
	textfield1->setBorderColor( border );
	textfield1->setBackColor( back );
	textfield1->setBaseColor( base );
	textfield1->setTextColor( fore );
	textfield1->setSelTextColor( selfore );
	textfield1->setSelBackColor( selback );
	textfield1->setCursorColor( fore );
	textfield1->setShadowColor( shadow );
	textfield1->setHiliteColor( hilite );
	
	tooltip->setTextColor( tipfore );
	tooltip->setBackColor( tipback );
	}

long FoxConfigurator::onColorChanged(FXObject * sender, FXSelector sel, void *)
{
	setupColors();
   	return 1;
}

void FoxConfigurator::saveLookAndFeel()
{
	
	FXchar buffer[ 60 ];
	settings.setBaseColor(fxnamefromcolor( buffer, base ));
	settings.setBackColor(fxnamefromcolor( buffer, back ));
	settings.setForeColor(fxnamefromcolor( buffer, fore ));
	settings.setHiliteColor(fxnamefromcolor( buffer, hilite ));
	settings.setShadowColor(fxnamefromcolor( buffer, shadow ));
	settings.setSelforeColor(fxnamefromcolor( buffer, selfore ));
	settings.setSelbackColor(fxnamefromcolor( buffer, selback ));
	settings.setTipforeColor(fxnamefromcolor( buffer, tipfore ));
	settings.setTipbackColor(fxnamefromcolor( buffer, tipback ));
	settings.setMenuforeColor(fxnamefromcolor( buffer, menufore ));
	settings.setMenubackColor(fxnamefromcolor( buffer, menuback ));
	settings.setMainColor(fxnamefromcolor( buffer, main ));
	
	settings.setFontDescription("normal",font[0]->getFont().text());
	settings.setFontDescription("big",font[1]->getFont().text());
	settings.setFontDescription("small",font[2]->getFont().text());
	
}


void FoxConfigurator::setupMainSettings()
{
	FXVerticalFrame *mainpane = new FXVerticalFrame ( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y );
	
	new FXButton ( buttons, "Main Settings", foxIconManager.get("main_settings"), switcher, FXSwitcher::ID_OPEN_FIRST, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
	
	FXHorizontalFrame *hoz2 = new FXHorizontalFrame ( mainpane, LAYOUT_FILL_X );
	new FXLabel ( hoz2, "width:" );
	mainwindow_width = new FXSpinner( hoz2, 4, this,ID_MAINWINDOWWIDTH );
	mainwindow_width->setRange( 1, 2000 );
	mainwindow_width->setIncrement( 1 );
	mainwindow_width->setValue( settings.getWindowWidth());
	new FXLabel ( hoz2, "height:" );
	mainwindow_height = new FXSpinner( hoz2, 4,this,ID_MAINWINDOWHEIGHT  );
	mainwindow_height->setRange( 1, 2000 );
	mainwindow_height->setIncrement( 1 );
	mainwindow_height->setValue( settings.getWindowHeight()  );
	
	saveWindowSizeCheck = new FXCheckButton ( mainpane, "save window size on close", NULL, 0, JUSTIFY_LEFT | JUSTIFY_TOP | ICON_BEFORE_TEXT | LAYOUT_SIDE_TOP );
	saveWindowSizeCheck->setCheck(settings.getSaveWindowSize());
	
	
  	new FXLabel ( mainpane, "maximum size of image file for generating thumbnails (in Bytes)" );
  	
	thumbsize = new FXSpinner( mainpane, 8 );
	thumbsize->setRange( 0, 10000000 );
	thumbsize->setIncrement( 1000 );
	thumbsize->setValue( static_cast<int>(settings.getMaxThumbSize()) );
	
	
	new FXLabel ( mainpane, "nao placement on startup" );
	placement = new FXListBox ( mainpane );
	placement->setNumVisible( 2 );
	placement->appendItem( "centered" );
	placement->appendItem( "cursor" );
	
	if(settings.getStartupPlacement()=="cursor")
  	 placement->setCurrentItem(1);
  	 
  	new FXLabel ( mainpane, "icons theme:" );
   	iconsTheme = new FXListBox ( mainpane );
   	iconsTheme->setNumVisible( 5 );
	
	
	vector<string> themes=settings.getAvailableIconsThemes();
	vector<string>::iterator iter;
	for(iter=themes.begin();iter!=themes.end();++iter)
	{
		iconsTheme->appendItem(iter->c_str() );
	}
	
	
	iconsTheme->setCurrentItem(iconsTheme->findItem(settings.getIconsTheme().c_str()));
	
	
	new FXSeparator( mainpane );

	new FXLabel ( mainpane, "left directory:" );
	leftdir = new FXTextField ( mainpane, 50 );
	leftdir->setText( settings.getLeftPath().c_str () );
	
	new FXLabel ( mainpane, "right directory (in single directory window mode this is default window):" );
	rightdir = new FXTextField ( mainpane, 50 );
	rightdir->setText( settings.getRightPath().c_str ()  );
	
	new FXLabel ( mainpane, "default new opened directory:" );
	defaultdir = new FXTextField ( mainpane, 50 );
	defaultdir->setText( settings.getDefaultPath().c_str ()  );	
	
	
	showHiddenFilesCheck = new FXCheckButton ( mainpane, "show hidden files", NULL, 0, JUSTIFY_LEFT | JUSTIFY_TOP | ICON_BEFORE_TEXT | LAYOUT_SIDE_TOP );
	showHiddenFilesCheck->setCheck(settings.getShowHiddenFiles());
	
	
	
	new FXLabel ( mainpane, "maximum files in history" );
  	
	historySize = new FXSpinner( mainpane, 8 );
	historySize->setRange( 0, 200 );
	historySize->setIncrement( 1 );
	historySize->setValue( static_cast<int>(settings.getFilesHistorySize()) );
	
}


void FoxConfigurator::saveMainSettings()
{
	settings.setFilesHistorySize(historySize->getValue());
	settings.setMaxThumbSize(thumbsize->getValue());
	settings.setWindowHeight(mainwindow_height->getValue());
	settings.setWindowWidth(mainwindow_width->getValue());
	settings.setStartupPlacement(placement->getItem(placement->getCurrentItem()).text());
	settings.setIconsTheme(iconsTheme->getItem(iconsTheme->getCurrentItem()).text());
	settings.setLeftPath(leftdir->getText().text());
	settings.setRightPath(rightdir->getText().text());
	settings.setDefaultPath(defaultdir->getText().text());
	
	settings.setShowHiddenFiles(showHiddenFilesCheck->getCheck());
	settings.setSaveWindowSize(saveWindowSizeCheck->getCheck());
	
}

long FoxConfigurator::onChooseFont(FXObject * sender, FXSelector sel, void *)
{
    FXushort id = FXSELID( sel );
    int nr = id - ID_CHOOSE_FONT1;


    FXFontDialog dialog( this, "Select Font" );
    FXFontDesc fontdescription;
    font[ nr ] ->getFontDesc( fontdescription );
    strncpy( fontdescription.face, font[ nr ] ->getActualName().text(), sizeof( fontdescription.face ) );
    dialog.setFontSelection( fontdescription );
    if ( dialog.execute( PLACEMENT_SCREEN ) )
    {
        FXFont * oldfont = font[ nr ];
        dialog.getFontSelection( fontdescription );


        font[ nr ] = new FXFont( getApp(), fontdescription );
        font[ nr ] ->create();

        setupFont( nr );
        delete oldfont;

    }
    return 1;
}

void FoxConfigurator::setupFont( int nr )
{

    FXString fontname = font[ nr ] ->getFont();
    
    if ( nr == 0 )
    {

        tabitem->setFont( font[ nr ] );
        label1->setFont( font[ nr ] );
        label2->setFont( font[ nr ] );
        label3->setFont( font[ nr ] );
        label4->setFont( font[ nr ] );
        label5->setFont( font[ nr ] );

        menulabels[ 0 ] ->setFont( font[ nr ] );
        menulabels[ 1 ] ->setFont( font[ nr ] );
        menulabels[ 2 ] ->setFont( font[ nr ] );
        menulabels[ 3 ] ->setFont( font[ nr ] );
        menulabels[ 4 ] ->setFont( font[ nr ] );
        menulabels[ 5 ] ->setFont( font[ nr ] );

        textfield1->setFont( font[ nr ] );

    }

    fontbutton[ nr ] ->setText( fontname );

}

void FoxConfigurator::setupVfs()
{
	FXVerticalFrame *vfsPane = new FXVerticalFrame ( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	new FXLabel ( vfsPane, "VFS settings", NULL, LAYOUT_LEFT );
	
	new FXButton ( buttons, "Virtual File System Settings", foxIconManager.get("vfs_settings"), switcher, FXSwitcher::ID_OPEN_SIXTH, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
	
	vfsList = new FXListBox ( vfsPane, this, ID_VFS_CHANGE );

	vector<string> allVfs=settings.getAllVfs();
	vector<string>::iterator iter;
	for(iter=allVfs.begin();iter!=allVfs.end();++iter)
	{
		vfsList->appendItem(iter->c_str());
	}	
	
	allVfs.size()<20 ? vfsList->setNumVisible(allVfs.size()+1): vfsList->setNumVisible( 20 );
	
	FXHorizontalFrame *hzframe = new FXHorizontalFrame ( vfsPane, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	FXVerticalFrame *vframe0 = new FXVerticalFrame ( hzframe, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	new FXLabel( vframe0, "visible headers:" );
	headersList = new FXList ( vframe0, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
	headersList->setNumVisible( 5 );
	FXVerticalFrame *vframe = new FXVerticalFrame ( hzframe, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	new FXLabel( vframe, "\n" );
	new FXArrowButton( vframe, this, ID_UP_VFS_HEADER, FRAME_RAISED | FRAME_THICK | ARROW_UP );
	new FXArrowButton( vframe, this, ID_DOWN_VFS_HEADER, FRAME_RAISED | FRAME_THICK | ARROW_DOWN );
	new FXArrowButton( vframe, this, ID_ADD_HEADER, FRAME_RAISED | FRAME_THICK | ARROW_LEFT );
	new FXArrowButton( vframe, this, ID_DEL_HEADER, FRAME_RAISED | FRAME_THICK | ARROW_RIGHT );
	
	
	
	FXVerticalFrame *vframe1 = new FXVerticalFrame ( hzframe, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	new FXLabel( vframe1, "all available headers:" );
	availableHeadersList = new FXList ( vframe1, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
	availableHeadersList->setNumVisible( 5 );
    
	setActiveVfs(*allVfs.begin());

    

}

void FoxConfigurator::saveActiveVfs()
{
	
	vector<string> headers;
	
	for ( int i = 0; i < headersList->getNumItems (); i++ )
	{
		headers.push_back( headersList->getItemText( i ).text() );
	}
	
	settings.setHeaders(activeVfs,headers);
	
	
	string newActiveVfs=vfsList->getItem ( vfsList->getCurrentItem () ).text ();
	setActiveVfs(newActiveVfs);

}

void FoxConfigurator::setActiveVfs(string vfsName)
{
	vector<string> headers=settings.getHeaders(vfsName);
	vector<string>::iterator iter;
	
	headersList->clearItems();
	
	for(iter=headers.begin();iter!=headers.end();++iter)
	{
		headersList->appendItem(iter->c_str());
	}
	
	vector<string> availableHeaders=settings.getAvailableHeaders(vfsName);

	
	availableHeadersList->clearItems();
	
	for(iter=availableHeaders.begin();iter!=availableHeaders.end();++iter)
	{
		availableHeadersList->appendItem(iter->c_str());
	}
	
	
	activeVfs=vfsName;
}

long FoxConfigurator::onActiveVfsChange(FXObject * sender, FXSelector sel, void *)
{
	saveActiveVfs();
}

long FoxConfigurator::onUpDownVfsHeader(FXObject * sender, FXSelector sel, void *)
{
	if ( headersList->getNumItems () == 0 )
	return 0;
	
	FXushort id = FXSELID( sel );
	if ( id == ID_UP_VFS_HEADER )
	{
		if ( headersList->getCurrentItem() != 0 )
		headersList->moveItem( headersList->getCurrentItem() - 1, headersList->getCurrentItem() );
	}
	else
	{
		if ( headersList->getCurrentItem() != headersList->getNumItems () - 1 )
		headersList->moveItem( headersList->getCurrentItem() + 1, headersList->getCurrentItem() );
	}
	

}

long FoxConfigurator::onAddDelHeader(FXObject * sender, FXSelector sel, void *)
{
	FXushort id = FXSELID( sel );
	
	if ( id == ID_ADD_HEADER )
	{
		headersList->appendItem( availableHeadersList->getItemText( availableHeadersList->getCurrentItem() ) );
	}
	else
	{
		headersList->removeItem( headersList->getCurrentItem() );
	}
}

void FoxConfigurator::setupMenuAndButtons()
{

	FXVerticalFrame *buttonsPane = new FXVerticalFrame ( switcher, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	new FXButton ( buttons, "Menu and Buttons Settings", foxIconManager.get("menu_and_buttons"), switcher, FXSwitcher::ID_OPEN_SEVENTH, FRAME_RAISED | ICON_ABOVE_TEXT | LAYOUT_FILL_Y );
	
	new FXLabel ( buttonsPane, "Buttons in configurable toolbox:", NULL, LAYOUT_LEFT );
	
	toolbarList = new FXListBox ( buttonsPane, this, ID_TOOLBAR_CHANGE );
	toolbarList->setNumVisible( 10 );
	
	FXHorizontalFrame *hfx = new FXHorizontalFrame ( buttonsPane, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	buttonsList = new FXList ( hfx, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
	buttonsList->setNumVisible( 5 );
	
	new FXLabel( hfx, " " );
	new FXArrowButton( hfx, this, ID_UP_BUTTON_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_UP );
	new FXArrowButton( hfx, this, ID_DOWN_BUTTON_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_DOWN );
	
	/*TODO*/
	
	
	
	
	
	toolbarList->setCurrentItem ( toolbarList->getNumItems () - 1 );
	
	FXHorizontalFrame *buttonsHframe = new FXHorizontalFrame ( buttonsPane, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	new FXLabel( buttonsHframe, " " );
	new FXLabel( buttonsHframe, " " );
	new FXArrowButton( buttonsHframe, this, ID_ADD_TOOLBAR_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_UP );
	new FXArrowButton( buttonsHframe, this, ID_DEL_TOOLBAR_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_DOWN );
	
	
	new FXLabel ( buttonsPane, "All available commands:", NULL, LAYOUT_LEFT );
	
	additionalCommandsAvailableForButtons = new FXList ( buttonsPane, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
	additionalCommandsAvailableForButtons->setNumVisible( 8 );
//	additionalCommandsAvailableForButtons->appendItem( "SEPARATOR" );
	
	vector<string>::iterator iter;
	for(iter=commands.begin();iter!=commands.end();++iter)
    	{
    		additionalCommandsAvailableForButtons->appendItem( iter->c_str (),foxIconManager.get(settings.getCommandIcon( *iter )) );    			
    	}
	
	
	
	FXHorizontalFrame *buttonsHframe2 = new FXHorizontalFrame ( buttonsPane, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	new FXLabel( buttonsHframe2, " " );
	new FXLabel( buttonsHframe2, " " );
	new FXArrowButton( buttonsHframe2, this, ID_DEL_SHUTTER_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_UP );
	new FXArrowButton( buttonsHframe2, this, ID_ADD_SHUTTER_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_DOWN );
	new FXLabel ( buttonsPane, "Shutter:", NULL, LAYOUT_LEFT );
	
	
	shutterList = new FXListBox ( buttonsPane, this, ID_SHUTTER_CHANGE );
	shutterList->setNumVisible( 10 );
	
	hfx = new FXHorizontalFrame ( buttonsPane, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	
	
	shutterCommands = new FXList ( hfx, NULL, 0, LIST_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 250 );
	shutterCommands->setNumVisible( 5 );
	
	new FXLabel( hfx, " " );
	
	new FXArrowButton( hfx, this, ID_UP_SHUTTER_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_UP );
	new FXArrowButton( hfx, this, ID_DOWN_SHUTTER_COMMAND, FRAME_RAISED | FRAME_THICK | ARROW_DOWN );
	
	
	new FXSeparator( buttonsPane );
	FXHorizontalFrame *hfr = new FXHorizontalFrame ( buttonsPane, LAYOUT_FILL_X | LAYOUT_FILL_Y );
	new FXButton ( hfr, "Remove shutter", foxIconManager.get("minus"), this, ID_REMOVE_SHUTTER, FRAME_RAISED|ICON_BEFORE_TEXT  );
	new FXButton ( hfr, "Set default", NULL, this,ID_SET_DEFAULT_SHUTTER , FRAME_RAISED|ICON_BEFORE_TEXT  );
	new FXButton ( hfr, "New shutter", foxIconManager.get("plus"), this, ID_NEW_SHUTTER, FRAME_RAISED|ICON_BEFORE_TEXT  );
	newShutterEdit = new FXTextField ( hfr, 20 );
	
	
	vector<string>toolbars=settings.getToolbars();

	for(iter=toolbars.begin();iter!=toolbars.end();++iter)
	{
		toolbarList->appendItem(iter->c_str());
	}
	toolbars.size()<20 ? toolbarList->setNumVisible(toolbars.size()+1): toolbarList->setNumVisible( 20 );	

    	setActiveToolbar(*toolbars.begin());
    	
	
	
	vector<string>shutters=settings.getShutters();

	for(iter=shutters.begin();iter!=shutters.end();++iter)
	{
		shutterList->appendItem(iter->c_str());
	}
	shutters.size()<20 ? shutterList->setNumVisible(shutters.size()+1): shutterList->setNumVisible( 20 );	

    	setActiveShutter(*shutters.begin());	
	
	
}

long FoxConfigurator::onUpDownShuttersAndToolbars(FXObject * sender, FXSelector sel, void *)
{
	FXList * updownlist;
	FXListBox * box;
	bool up;
	
	FXushort id = FXSELID( sel );
	if ( id == ID_UP_BUTTON_COMMAND )
	{
		updownlist = buttonsList;
		box = toolbarList;
		up = true;
	}
	else if ( id == ID_DOWN_BUTTON_COMMAND )
	{
		updownlist = buttonsList;
		box = toolbarList;
		up = false;
	}
	else if ( id == ID_UP_SHUTTER_COMMAND )
	{
		updownlist = shutterCommands;
		box = shutterList;
		up = true;
	}
	else if ( id == ID_DOWN_SHUTTER_COMMAND )
	{
		updownlist = shutterCommands;
		box = shutterList;
		up = false;
	}
	
	if ( updownlist->getNumItems () == 0 )
		return 0;
	
	if ( up )
	{
		if ( updownlist->getCurrentItem() != 0 )
		updownlist->moveItem( updownlist->getCurrentItem() - 1, updownlist->getCurrentItem() );
	}
	else
	{
		if ( updownlist->getCurrentItem() != updownlist->getNumItems () - 1 )
		updownlist->moveItem( updownlist->getCurrentItem() + 1, updownlist->getCurrentItem() );
	}

}

void FoxConfigurator::setActiveShutter(string newActiveShutter)
{

	shutterCommands->clearItems();
	vector<string>commands=settings.getShutter(newActiveShutter);
	vector<string>::iterator iter;
	for(iter=commands.begin();iter!=commands.end();++iter)
	{
		shutterCommands->appendItem(iter->c_str(),foxIconManager.get(settings.getCommandIcon( *iter )));
	}
	
	activeShutter=newActiveShutter;
	

}

void FoxConfigurator::saveActiveShutter( )
{
	vector<string> commands;
	
	for ( int i = 0; i < shutterCommands->getNumItems (); i++ )
	{
		commands.push_back( shutterCommands->getItemText( i ).text() );
	}
	
	settings.setShutter(activeShutter,commands);
	
	
	string newActiveShutter=shutterList->getItem ( shutterList->getCurrentItem () ).text ();
	setActiveShutter(newActiveShutter);

}

void FoxConfigurator::saveActiveToolbar( )
{
	vector<string> commands;
	
	for ( int i = 0; i < buttonsList->getNumItems (); i++ )
	{
		commands.push_back( buttonsList->getItemText( i ).text() );
	}
	
	settings.setToolbarCommands(activeToolbar,commands);
	
	
	string newActiveToolbar=toolbarList->getItem ( toolbarList->getCurrentItem () ).text ();
	setActiveToolbar(newActiveToolbar);
}

void FoxConfigurator::setActiveToolbar(string newActiveToolbar)
{
	buttonsList->clearItems();
	vector<string>commands=settings.getToolbarCommands(newActiveToolbar);
	vector<string>::iterator iter;
	for(iter=commands.begin();iter!=commands.end();++iter)
	{
		buttonsList->appendItem(iter->c_str(),foxIconManager.get(settings.getCommandIcon( *iter )));
	}
	
	activeToolbar=newActiveToolbar;

}

long FoxConfigurator::onShutterChange(FXObject * sender, FXSelector sel, void *)
{
	saveActiveShutter();
	return 1;
}

long FoxConfigurator::onToolbarChange(FXObject * sender, FXSelector sel, void *)
{
	saveActiveToolbar();
	return 1;
}

long FoxConfigurator::onAddRemoveShutterCommand(FXObject * sender, FXSelector sel, void *)
{
	FXushort id = FXSELID( sel );
	
	if ( id == ID_ADD_SHUTTER_COMMAND )
	{
		string cmd = additionalCommandsAvailableForButtons->getItemText( additionalCommandsAvailableForButtons->getCurrentItem() ).text();
		shutterCommands->appendItem( cmd.c_str(),foxIconManager.get(settings.getCommandIcon( cmd )) );
	
	}
	else
	{
		if(shutterCommands->getCurrentItem()!=-1)
		shutterCommands->removeItem( shutterCommands->getCurrentItem() );
	}

}

long FoxConfigurator::onAddRemoveToolbarCommand(FXObject * sender, FXSelector sel, void *)
{
	FXushort id = FXSELID( sel );
	
	if ( id == ID_ADD_TOOLBAR_COMMAND )
	{
		string cmd = additionalCommandsAvailableForButtons->getItemText( additionalCommandsAvailableForButtons->getCurrentItem() ).text();
		buttonsList->appendItem( cmd.c_str(),foxIconManager.get(settings.getCommandIcon( cmd )) );
	
	}
	else
	{
		if(buttonsList->getCurrentItem()!=-1)
		buttonsList->removeItem( buttonsList->getCurrentItem() );
	}

}

long FoxConfigurator::onNewShutter(FXObject * sender, FXSelector sel, void *)
{
	string shutterName = newShutterEdit->getText ().text ();
	if(shutterName!="")
	{
	shutterList->appendItem( shutterName.c_str() );
	saveActiveShutter();
	}
}

long FoxConfigurator::onRemoveShutter(FXObject * sender, FXSelector sel, void *)
{
	if(shutterList->getCurrentItem ()!=-1)
	{
		string shutterName = shutterList->getItem ( shutterList->getCurrentItem () ).text ();
		int cur = shutterList->getCurrentItem ();
		shutterList->removeItem ( cur );
		settings.removeShutter(shutterName);
		
		if(shutterList->getNumItems()>0)
		setActiveShutter(shutterList->getItem (0).text());
	}
	
	
}

long FoxConfigurator::onSetDefaultShutter(FXObject * sender, FXSelector sel, void *)
{
	if(shutterList->getCurrentItem ()!=-1)
	{
	settings.setDefaultShutter(shutterList->getItem ( shutterList->getCurrentItem () ).text ());
	}
}

long FoxConfigurator::onCmdClose(FXObject * sender, FXSelector sel, void *)
{
	saveActiveCommand();
	saveActiveFileType();
	saveActiveVfs();
	saveSpeedAndDelays();
	saveLookAndFeel();
	saveMainSettings();
	saveActiveShutter();
	saveActiveToolbar();
	close(true);
}



