/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FOXFILELIST_H
#define FOXFILELIST_H

#include <pthread.h>

#include <fx.h>
#include "../FileListImpl.hpp"
#include "../FileListIcons.hpp"
#include "../FileIcon.hpp"
#include "../Settings.hpp"
#include "../Panel.hpp"
#include "FoxIconManager.hpp"
#include "FoxDockContainer.hpp"
#include "FoxStatus.hpp"
#include "FoxGuiThreadSynchronization.hpp"
#include "WindowContent.hpp"


#include <vector>

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/

 

class FoxFileList : public FXIconList, public FileListIcons, public SettingsObservator
{

FXDECLARE ( FoxFileList );

public:
    FoxFileList(FXComposite * p,FoxDockContainer *dock, Panel *panel);
    FoxFileList():settings(singleton<Settings>::instance()),foxIconManager(FoxSingleton<FoxIconManager>::instance(NULL)){}
    virtual ~FoxFileList();
      
    /*FileListIcons interface implementation*/  
      
    void addFileIcon(FileIcon fileIcon);
    void addFileIconSync(FileIcon fileIcon);
    
    void addFileIcons( vector<FileIcon> fileIcons);
    void addFileIconsSync(vector<FileIcon> fileIcons);
    
    void replaceFileIcon(int position,FileIcon);
    void replaceFileIconSync(int position,FileIcon);
    
    void removeFileIcon(int position);
    void removeFileIconSync(int position);
    
    FileIcon getFileIcon(int pos);
    FileIcon getFileIconSync(int pos);
    
    void clearItems(void);
    void clearItemsSync( void );
    
    bool isFileIconSelected(void);
    bool isFileIconSelectedSync(void);
    
    void selectFileIcon(int position);
    void selectFileIconSync(int position);
    
    vector<FileIcon>getSelectedFileIcons();
    vector<FileIcon>getSelectedFileIconsSync();
    
    void setHeaders(vector<Header>);
    void setHeadersSync(vector<Header>);
    	
    FileIcon getCurrentFileIcon();
    FileIcon getCurrentFileIconSync();
    
    Status* getStatusPanel(Cancelable *cancelable );
    /*doen't use in t hread*/
    
    void setView(string view);
    void setViewSync(string view);
    
    void setTextFieldList(vector<string>);
    void setTextFieldListSync(vector<string>);
    
    void setTextFieldCommands(vector<string>);
    /* ? */
    
    void setStatusInfo(string);
    void setStatusInfoSync(string);
    
    void switchPanelsView(void);
    void switchPanelsViewSync(void);
    
    void setFocus();
    void setFocusSync();
    
    void selectAll();
    void selectAllSync();
    
    void startProcessing();
    void startProcessingSync();
    
    void stopProcessing();
    void stopProcessingSync();
    
    int loadPreviewIcon(string file, int size);
    /* own mutex*/
    
    void unloadPreviewIcons();
    /* own mutex*/
    
    void disable();
    void disableSync();
    
    void enable();
    void enableSync();
    
    bool isProcessing();
    /* not needed*/
      
    
    void createPanel(string name, string param );
    void createPanelSync(string name, string param );
    
    void ask(WindowContent *windowContent,map<string,string> &result);
    void askSync(Wait *w,WindowContent *windowContent);
    
    void clipboardCopy(bool cut=false);
    void clipboardCopySync(bool cut=false);
    
    void clipboardPaste();
    void clipboardPasteSync();
    
    /*****************************************/  
    
    /*SettingsObservator interface implementation*/ 
    
    void notify();
    void notifySync();
    
    /*****************************************/  
    
    
    
    void openPath(string);
    vector<string> getDirectories(void);
    void hideToolbars();
    void showToolbars();
    void deleteToolbars();
    void makeToolbars();    
    void lostFocus ();  
    void setOpposite(FoxFileList* theValue);
    
   
    virtual void create ();
    

    
    enum
    {
        ID_FILE = FXIconList::ID_LAST,
        ID_SORT_CHANGE,
        ID_TEXT_FIELD,
        ID_TEXT_FIELD_SWITCH,
        ID_TIMER,
        ID_STOP_EXECUTION,
        ID_OPENTIMER,
        ID_LAST,
    };    


	long onOpenFile ( FXObject * obj, FXSelector sel, void *ptr );
    	long onUpDir ( FXObject * obj, FXSelector sel, void *ptr );  
    	long onCommand ( FXObject * obj, FXSelector sel, void *ptr );  
    	long onFocus ( FXObject * obj, FXSelector sel, void *ptr ); 
	long onClick ( FXObject * obj, FXSelector sel, void *ptr );
	long onPopup ( FXObject * obj, FXSelector sel, void *ptr );
	long onSort ( FXObject * obj, FXSelector sel, void *ptr );
	long onTextFieldExecute ( FXObject * obj, FXSelector sel, void *ptr );
	long onTextFieldSwitch ( FXObject * obj, FXSelector sel, void *ptr );
	long onCmdResizeHeader ( FXObject * obj, FXSelector sel, void *ptr );
	long onMouseButtonPress ( FXObject * obj, FXSelector sel, void *ptr );
	long onMotion ( FXObject * obj, FXSelector sel, void *ptr );
	long onMouseButtonRelease ( FXObject * obj, FXSelector sel, void *ptr );
	long onFocusNext ( FXObject * obj, FXSelector sel, void *ptr );
	long onTimer ( FXObject * obj, FXSelector sel, void *ptr );
	long onStopExecution ( FXObject * obj, FXSelector sel, void *ptr );
	long onOpenTimer(FXObject*,FXSelector,void*);
	long onRightButtonPress(FXObject*,FXSelector,void*);


	
	//drag and drop functions
	long onDNDEnter ( FXObject *, FXSelector, void * );
	long onDNDLeave ( FXObject *, FXSelector, void * );
	long onDNDMotion ( FXObject *, FXSelector, void * );
	long onDNDDrop ( FXObject *, FXSelector, void * );
	long onDNDRequest ( FXObject *, FXSelector, void * );
	long onBeginDrag ( FXObject *, FXSelector, void * );
	long onEndDrag ( FXObject *, FXSelector, void * );
	long onDragged ( FXObject *, FXSelector, void * );	
	long onClipboardLost ( FXObject *, FXSelector, void * );
	long onClipboardGained ( FXObject *, FXSelector, void * );
	long onClipboardRequest ( FXObject *, FXSelector, void * );

	

	
	FileList* getFileList() const
	{
	    return fileList;
	}

	Panel* getPanel() const
	{
	    return panel;
	}
	
	
	
    	 
    

private:
	
	
	void setShorcuts();
	void makeToolbar(string name);
	void setDraggedFiles();
	void dropData ( bool clipboard );
	
	FXDragAction dropaction;
	
	FXLabel* statusInfo;
	map<string, unsigned int > keyMap;
	FileList *fileList;
	FileList *fileListImpl;
	FoxIconManager &foxIconManager;
	Panel *panel;
	FXApp *app;
	FoxDockContainer *dock;
	Settings& settings;
	vector<string>commandsVector;
	FXToolBar* toolbar;
	FXToolBar *toolbar2;
	FXToolBarShell *dragshell;
	FXToolBarShell *dragshell2;
	FXDockSite *docksite;
	FXButton *executionButton;
	
	FoxFileList *opposite;
	bool constructor;
	vector<FXButton*>buttons;
	FXMenuPane *popupMenu;
	FXString dragfiles;
	FXComboBox *textField;
	FXPopup *sortpop;
	FXOptionMenu *sortMenu;
	FXPopup *textFieldPop;
	FXOptionMenu *textFieldMenu;
	
	bool mousePressed;
	int tabCommandNr;
	
	vector<FoxStatus*>statusVector;
	
	pthread_t p_id;
	FXGUISignal *guiSignal;
	boost::mutex mutexS;
	FoxSingalReceiver foxSingalReceiver;
	boost::mutex muteX;
	boost::mutex muteX2;
	
	bool processing;
	int rotateAngel;
	vector<FXIcon*> previewIcons;
	FXIconSource *source;

	bool mouseGestureActive;
	bool pressedRightButton;
	int lastX, lastY;
	vector<int>gests;
	
	bool active;
	bool lastAnyFileSelected;
	int delay;

};

#endif
