/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FoxFileListItem.hpp"
#include "FoxFonts.hpp"

FXIMPLEMENT( FoxFileListItem, FXIconItem, NULL, 0 );

// Constants definition
#define SIDE_SPACING             4    // Left or right spacing between items
#define DETAIL_TEXT_SPACING      2    // Spacing between text and icon in detail icon mode
#define MINI_TEXT_SPACING        2    // Spacing between text and icon in mini icon mode
#define BIG_LINE_SPACING         0    // Line spacing in big icon mode
#define BIG_TEXT_SPACING         0    // Spacing between text and icon in big icon mode
#define ITEM_SPACE             128    // Default space for item


FoxFileListItem::~FoxFileListItem()
{
}

FoxFileListItem::FoxFileListItem(FileIcon _fileIcon, FXIcon * icon1, FXIcon * icon2,FXApp *app,FXIcon *_extraIcon)
 :  FXIconItem( _fileIcon.getShowName().c_str(), icon1, icon2, NULL ), fileIcon(_fileIcon),extraIcon(_extraIcon)
{
	fields=fileIcon.getFile().getFields();
		
	string text=fileIcon.getShowName();
	vector<string>::iterator fieldIter;
	for(fieldIter=fields.begin();fieldIter!=fields.end();++fieldIter)
	{
		text+="\t"+*fieldIter;
	}
	setText(text.c_str());
	
	FoxFonts &foxFonts=FoxSingleton<FoxFonts>::instance(app);
	fontNormal=foxFonts.getFont("normal");
	fontBig=foxFonts.getFont("big");
	fontSmall=foxFonts.getFont("small");
	
	if(fileIcon.getColor()!="")
	{
	color=fxcolorfromname(fileIcon.getColor().c_str());
	}
	else
	{
	color=FXRGB(0,0,0);
	}
	
	
	if(fileIcon.getBackColor()!="")
	{
	backcolor=fxcolorfromname(fileIcon.getBackColor().c_str());
	}
	else
	{
	backcolor=FXRGB(255,255,255);
	}
	
	
	
}

void FoxFileListItem::drawBigIcon( const FXIconList* list, FXDC& dc, FXint x, FXint y, FXint w, FXint h ) const
{

	
	//=============CHANGES==========================================
	FXString label = fileIcon.getShowName().c_str();
	FXFont *font =  fontBig;
	
	
    register FXint len, dw, s, space, xt, yt, xi, yi;
    //register FXFont *font=list->getFont();
    register FXint iw = 0, ih = 0, tw = 0, th = 0, ss = 0;
    space = w - SIDE_SPACING;
    if ( !label.empty() )
    {
        for ( len = 0; len < label.length() && label[ len ] != '\t'; len++ );
        tw = 4 + font->getTextWidth( label.text(), len );




        if ( bigIcon )
        {
            ih = bigIcon->getHeight();
            iw = bigIcon->getWidth();
            xi = x + ( w - iw ) / 2;
        }




        th = 4 + font->getFontHeight();
        //=============CHANGES==========================================
        for ( int i = 0;i < fields.size();i++ )
        {
            th = th + 4 + fontSmall->getFontHeight();
        }
        //=============CHANGES==========================================


        //yt=y+h-th-BIG_LINE_SPACING/2;
        yi = y + BIG_LINE_SPACING / 2 + ( h - th - BIG_LINE_SPACING - ss - ih ) / 2;
        yt = y;

        dw = 0;
        if ( tw > space )
        {
            dw = font->getTextWidth( "...", 3 );
            s = space - dw;
            while ( ( tw = 4 + font->getTextWidth( label.text(), len ) ) > s && len > 1 ) --len;
            if ( tw > s ) dw = 0;
        }
        if ( tw <= space )
        {         // FIXME as below in drawDetails
            //xt=x+(w-tw-dw)/2;
            xt = x + iw;

            if ( isSelected() )
            {
                // dc.setForeground(list->getSelBackColor());
                //dc.fillRectangle(xt,yt,tw+dw,th);
            }
            dc.setForeground( backcolor );
            dc.fillRectangle( x, y, w, h );


            if ( !isEnabled() )
                dc.setForeground( makeShadowColor( list->getBackColor() ) );
            else if ( isSelected() )
                dc.setForeground( color );
            else
                dc.setForeground( color );


            dc.drawText( xt + 2 - iw, yt + font->getFontAscent() + 2, label.text(), len );


            if ( dw ) dc.drawText( xt - iw + tw - 2, yt + font->getFontAscent() + 2, "...", 3 );
            //=============CHANGES==========================================
            dc.setFont( fontSmall );
            
            
            fieldIter=fields.begin();

            for ( int i = 0;i < fields.size();i++ )
            {
		
                dc.drawText( xt + 4, yt + ( i + 2 ) * font->getFontAscent() + 2, fieldIter->c_str(), fieldIter->size() );
		fieldIter++;
            }
            //=============CHANGES==========================================
            dc.setFont( font );
            /*if(hasFocus()){
              dc.drawFocusRectangle(xt+1,yt+1,tw+dw-2,th-2);
              }*/
        }
        ss = 0; //BIG_TEXT_SPACING;    // Space between text and icon only added if we have both icon and text
    }
    if ( bigIcon )
    {

        //=============CHANGES==========================================
        //xi = x;
	xi=x+ 2;
        //=============CHANGES==========================================
        yi = y + BIG_LINE_SPACING / 2 + ( h - th - BIG_LINE_SPACING - ss - ih ) / 2;
        if ( isSelected() )        {
        	
            	dc.drawIconShaded( bigIcon, xi, y + 4 + font->getFontHeight() );
        }
        else
        {
            dc.drawIcon( bigIcon, xi, y + 4 + font->getFontHeight() );
        }
        
        if(extraIcon)
        {
            dc.drawIcon( extraIcon, xi, y + 4 + font->getFontHeight() );
        }   
           
        /*      
        
        if ( osf.content != "" )
        {
            dc.setFont( fontSmall );
            dc.drawText( xi, y + 4 + font->getFontHeight() + font->getFontAscent() + 2, osf.content.c_str(), 10 );
            dc.setFont( font );
        }
        */

    }
}


FXint FoxFileListItem::getHeight( const FXIconList* list ) const
{
    FXFont * font =list->getFont();

    register FXuint options = list->getListStyle();
    register FXint ih = 0, th = 0, h = 0, firstth = 0;
    if ( options & ICONLIST_BIG_ICONS )
    {
        if ( bigIcon ) ih = bigIcon->getHeight();
        if ( !label.empty() ) th = 4 + font->getFontHeight();
        if ( ih && th ) ih += BIG_TEXT_SPACING;
        firstth = th;
        
        //=============CHANGES==========================================        
        for ( int i = 0;i < fields.size();i++ )
        {
            th = th + 2 + fontSmall->getFontHeight();
        }        
        //=============CHANGES==========================================

        if ( ih + firstth > th )
            h = BIG_LINE_SPACING + ih + firstth + 4;
        else
            h = BIG_LINE_SPACING + th;

    }
    else if ( options & ICONLIST_MINI_ICONS )
    {
        if ( miniIcon ) ih = miniIcon->getHeight();
        if ( !label.empty() ) th = 4 + list->getFont() ->getFontHeight();
        h = FXMAX( ih, th );
    }
    else
    {
        if ( miniIcon ) ih = miniIcon->getHeight();
        if ( !label.empty() ) th = 4 + list->getFont() ->getFontHeight();
        h = FXMAX( ih, th );
    }

    return h;
}




// Draw mini icon
void FoxFileListItem::drawMiniIcon(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const {
  register FXFont *font=list->getFont();
  register FXint iw=0,ih=0,tw=0,th=0;
  register FXint len,dw,s,space;
  x+=SIDE_SPACING/2;
  space=w-SIDE_SPACING;
  
  dc.setForeground( backcolor );
  dc.fillRectangle( x, y, w, h );
  
  
  if(miniIcon){
    iw=miniIcon->getWidth();
    ih=miniIcon->getHeight();
    if(isSelected()){
      dc.drawIconShaded(miniIcon,x,y+(h-ih)/2);
      }
    else{
      dc.drawIcon(miniIcon,x,y+(h-ih)/2);
      }
    x+=iw+MINI_TEXT_SPACING;
    space-=iw+MINI_TEXT_SPACING;
    }
  if(!label.empty()){
    for(len=0; len<label.length() && label[len]!='\t'; len++);
    tw=4+font->getTextWidth(label.text(),len);
    th=4+font->getFontHeight();
    dw=font->getTextWidth("...",3);
    y+=(h-th)/2;
    dw=0;
    if(tw>space){                  // FIXME as below in drawDetails
      dw=font->getTextWidth("...",3);
      s=space-dw;
      while((tw=4+font->getTextWidth(label.text(),len))>s && len>1) 
      
      #ifdef  HAVE_LIBFOX_1_6
      len=label.dec(len);
      #else
      --len;
      #endif
      
      
      if(tw>s) dw=0;
      }
    if(tw<=space){
      if(isSelected()){
        dc.setForeground(list->getSelBackColor());
        dc.fillRectangle(x,y,tw+dw,th);
        }
      if(!isEnabled())
        dc.setForeground(makeShadowColor(list->getBackColor()));
      else if(isSelected())
        dc.setForeground(list->getSelTextColor());
      else
        //dc.setForeground(list->getTextColor());
        dc.setForeground(color);
      dc.drawText(x+2,y+font->getFontAscent()+2,label.text(),len);
      if(dw) dc.drawText(x+tw-2,y+font->getFontAscent()+2,"...",3);
      if(hasFocus()){
        dc.drawFocusRectangle(x+1,y+1,tw+dw-2,th-2);
        }
      }
    }
  }


// Draw detail
void FoxFileListItem::drawDetails(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint,FXint h) const {
  register FXHeader *header=list->getHeader();
  register FXFont *font=list->getFont();
  register FXint iw=0,ih=0,tw=0,th=0,yt,beg,end,hi,drw,space,used,dw,xx;
  if(header->getNumItems()==0) return;
  
  dc.setForeground( backcolor );
  dc.fillRectangle( x, y, header->getTotalSize(), h );
  
  
  if(isSelected()){
  
    dc.setForeground(list->getSelBackColor());
    //dc.setForeground(backcolor);
    
    dc.fillRectangle(x,y,header->getTotalSize(),h);
    }
  if(hasFocus()){
    dc.drawFocusRectangle(x+1,y+1,header->getTotalSize()-2,h-2);
    }
  xx=x+SIDE_SPACING/2;
  if(miniIcon){
    iw=miniIcon->getWidth();
    ih=miniIcon->getHeight();
    dc.setClipRectangle(x,y,header->getItemSize(0),h);
    dc.drawIcon(miniIcon,xx,y+(h-ih)/2);
    dc.clearClipRectangle();
    xx+=iw+DETAIL_TEXT_SPACING;
    }
  if(!label.empty()){
    th=font->getFontHeight();
    dw=font->getTextWidth("...",3);
    yt=y+(h-th-4)/2;
    if(!isEnabled())
      dc.setForeground(makeShadowColor(list->getBackColor()));
    else if(isSelected())
      dc.setForeground(list->getSelTextColor());
    else
      //dc.setForeground(list->getTextColor());
      dc.setForeground(color);
    used=iw+DETAIL_TEXT_SPACING+SIDE_SPACING/2;
    for(hi=beg=0; beg<label.length() && hi<header->getNumItems(); hi++,beg=end+1){
      space=header->getItemSize(hi)-used;
      for(end=beg; end<label.length() && label[end]!='\t'; end++);
      if(end>beg){
        drw=end-beg;
        tw=font->getTextWidth(&label[beg],drw);
        if(tw>space-4){
          while((tw=font->getTextWidth(&label[beg],drw))+dw>space-4 && drw>1) 
	  
	 #ifdef  HAVE_LIBFOX_1_6
      	 drw=label.dec(drw);
     	 #else
      	 --drw;
     	 #endif
	  
	  
          dc.setClipRectangle(xx,y,space,h);
          dc.drawText(xx+2,yt+font->getFontAscent()+2,&label[beg],drw);
          dc.drawText(xx+tw+2,yt+font->getFontAscent()+2,"...",3);
          dc.clearClipRectangle();
          }
        else{
          dc.drawText(xx+2,yt+font->getFontAscent()+2,&label[beg],drw);
          }
        }
      xx+=space;
      used=0;
      }
    }
  }






FXint FoxFileListItem::hitItem( const FXIconList* list, FXint rx, FXint ry, FXint rw, FXint rh ) const
{

    FXFont * font =list->getFont();

    register FXint iw = 0, tw = 0, ih = 0, th = 0, ss = 0, ix, iy, tx, ty, w, h, sp, tlen;
    register FXuint options = list->getListStyle();
    //register FXFont *font=list->getFont();
    for ( tlen = 0; tlen < label.length() && label[ tlen ] != '\t'; tlen++ );
    if ( options & ICONLIST_BIG_ICONS )
    {
        w = list->getItemSpace();
        h = list->getItemHeight();
        sp = w - SIDE_SPACING;
        if ( !label.empty() )
        {
            tw = 4 + font->getTextWidth( label.text(), tlen );
            th = 4 + font->getFontHeight();
            if ( tw > sp ) tw = sp;
            if ( bigIcon ) ss = BIG_TEXT_SPACING;
        }
        if ( bigIcon )
        {
            iw = bigIcon->getWidth();
            ih = bigIcon->getHeight();
        }
        ty = h - th - BIG_LINE_SPACING / 2;
        iy = 4 + font->getFontHeight();
        ix = 0;
        tx = ( w - tw ) / 2;
    }
    else if ( options & ICONLIST_MINI_ICONS )
    {
        sp = list->getItemSpace() - SIDE_SPACING;
        ix = SIDE_SPACING / 2;
        tx = SIDE_SPACING / 2;
        if ( miniIcon )
        {
            iw = miniIcon->getWidth();
            ih = miniIcon->getHeight();
            tx += iw + MINI_TEXT_SPACING;
            sp = sp - iw - MINI_TEXT_SPACING;
        }
        if ( !label.empty() )
        {
            tw = 4 + font->getTextWidth( label.text(), tlen );
            th = 4 + font->getFontHeight();
            if ( tw > sp ) tw = sp;
        }
        h = list->getItemHeight();
        iy = ( h - ih ) / 2;
        ty = ( h - th ) / 2;
    }
    else
    {
        ix = SIDE_SPACING / 2;
        tx = SIDE_SPACING / 2;
        if ( miniIcon )
        {
            iw = miniIcon->getWidth();
            ih = miniIcon->getHeight();
            tx += iw + DETAIL_TEXT_SPACING;
        }
        if ( !label.empty() )
        {
            tw = 10000000;
            th = 4 + font->getFontHeight();
        }
        h = list->getItemHeight();
        iy = ( h - ih ) / 2;
        ty = ( h - th ) / 2;
    }

    // In icon?
    if ( ix <= rx + rw && iy <= ry + rh && rx < ix + iw && ry < iy + ih ) return 1;

    // In text?
    if ( tx <= rx + rw && ty <= ry + rh && rx < tx + tw && ry < ty + th ) return 2;

    // Outside
    return 0;
}


