/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FoxInputDialog.hpp"
#include "FoxIconManager.hpp"

#include <locale.h> 
#include <libintl.h>
#define _(STRING)    gettext(STRING)

#include "fxkeys.h"

#include <iostream>
using namespace std;


FXDEFMAP(FoxInputDialog) FoxInputDialogMap[]=
{
	FXMAPFUNC(SEL_COMMAND,FoxInputDialog::ID_CLICKED_BUTTON,FoxInputDialog::onCmdClicked),
	FXMAPFUNC(SEL_COMMAND,FoxInputDialog::ID_PRESSED_ENTER,FoxInputDialog::onPressedEnter),
	FXMAPFUNC(SEL_CLOSE,0,FoxInputDialog::onCmdClicked),
};

FXIMPLEMENT(FoxInputDialog,FXDialogBox,FoxInputDialogMap,ARRAYNUMBER(FoxInputDialogMap));



FoxInputDialog::~FoxInputDialog()
{

}

FoxInputDialog::FoxInputDialog(FXWindow * owner, Wait *_w,WindowContent *windowContent)
 : FXDialogBox ( owner, windowContent->getTitle().c_str(), DECOR_ALL, 0, 0, 0/*windowContent->getWidth()*/,0/*windowContent->getHeight()*/, 0, 0, 0, 0, 4, 4 ), w(_w)
{

	try
	{
		while(1)
		{
				
			Widget *widget=windowContent->getNext();			
		
			defaultButton=windowContent->getDefaultButton();
			
			FXComposite *parentComposite;
			FXWindow *window;
			
			if(widget->getParentName()=="")
			{
				parentComposite=this;
			}
			else
			{
				parentComposite=dynamic_cast<FXComposite*>(widgetsMap[widget->getParentName()]);
			}
			
			if(widget->getType()=="VerticalFrame")
			{
				window=new FXVerticalFrame(parentComposite,LAYOUT_FILL_X|LAYOUT_FILL_Y);				
			}
			else if(widget->getType()=="HorizontalFrame")
			{
				window=new FXHorizontalFrame(parentComposite,LAYOUT_FILL_X|LAYOUT_FILL_Y);
			}
			else if(widget->getType()=="GroupBox")
			{
				WidgetGroupBox *widget2=dynamic_cast<WidgetGroupBox*>(widget);
				window= new FXGroupBox (parentComposite,widget2->getValue().c_str() , GROUPBOX_TITLE_LEFT | FRAME_GROOVE | LAYOUT_FILL_X);
			}
			else if(widget->getType()=="CheckButton")
			{
				WidgetCheckButton *widget2=dynamic_cast<WidgetCheckButton*>(widget);
				FXCheckButton *check= new FXCheckButton (parentComposite, widget2->getLabel().c_str());
				if(widget2->getValue())
					check->setCheck();
				window=check;
			}
			else if(widget->getType()=="Label")
			{
				WidgetLabel *widget2=dynamic_cast<WidgetLabel*>(widget);
				window=new FXLabel(parentComposite,widget2->getValue().c_str(),FoxSingleton<FoxIconManager>::instance(getApp()).get(widget2->getIconName()),ICON_BEFORE_TEXT);
			
				
			}
			else if(widget->getType()=="TextField")
			{
				WidgetTextField *widget2=dynamic_cast<WidgetTextField*>(widget);
				int style=0;
				if(widget2->getStyle().find("encrypt")!=string::npos)
				style|=TEXTFIELD_PASSWD;
				else if(widget2->getStyle().find("integer")!=string::npos)
				style|=TEXTFIELD_INTEGER;
				else if(widget2->getStyle().find("readonly")!=string::npos)
				style|=TEXTFIELD_READONLY;
				
				style=style|TEXTFIELD_ENTER_ONLY;
				
				FXTextField *textField=new FXTextField (parentComposite, widget2->getFieldsNum(),this,ID_PRESSED_ENTER,FRAME_THICK|style);
				textField->setText(widget2->getValue().c_str());
				
				if(widget2->getStyle().find("select")!=string::npos)
				textField->selectAll();
				
				window=textField;
				window->setFocus();
				
			}
			else if(widget->getType()=="Button")
			{
				WidgetButton *widget2=dynamic_cast<WidgetButton*>(widget);
				window=new FXButton (parentComposite,widget2->getValue().c_str(),FoxSingleton<FoxIconManager>::instance(getApp()).get(widget2->getIconName()),this,ID_CLICKED_BUTTON,BUTTON_INITIAL|FRAME_RAISED|ICON_BEFORE_TEXT | JUSTIFY_LEFT);
			}
			else if(widget->getType()=="ComboBox")
			{
				WidgetComboBox *widget2=dynamic_cast<WidgetComboBox*>(widget);
				FXComboBox *listBox=new FXComboBox (parentComposite,widget2->getFieldsNum(),NULL,0,COMBOBOX_INSERT_BEFORE|FRAME_THICK);
				listBox->setNumVisible( 5 );
				
				vector<string>::iterator iter;
				vector<string> values=widget2->getValues();
				for(iter=values.begin();iter!=values.end();++iter)
				{
					listBox->appendItem(iter->c_str());
				}
				
				window=listBox;
			
			}
			
			widgetsMap[widget->getName()]=window;
			widgetsVec.push_back(FoxWidget(widget->getName(),widget->getType(),window));
		
		}
	}
	catch(exception e){}

	if(windowContent->getWidgetFocus()!="")
	widgetsMap[windowContent->getWidgetFocus()]->setFocus();


}

long FoxInputDialog::onCmdClicked(FXObject *obj, FXSelector sel, void *)
{
	boost::mutex::scoped_lock lock(*w->m_mutex);
	
	FXWindow *button;
	
 	if(FXSELTYPE(sel)==SEL_CLOSE)
	{
		(*w->result)["pressedButton"]="cancel";
	}
	else
	{
		button=dynamic_cast<FXWindow*>(obj);
	}
	
	
	vector<FoxWidget>::iterator iter;
	for(iter=widgetsVec.begin();iter!=widgetsVec.end();++iter)
	{
	
		if(button==iter->getFoxWidget())
		{
		(*w->result)["pressedButton"]=iter->getName();
		continue;
		}
	
		string type=iter->getType();
		if(type=="TextField")
		{
			FXTextField *textField=dynamic_cast<FXTextField*>(iter->getFoxWidget());
			(*w->result)[iter->getName()]=textField->getText().text();
		}
		else if(type=="CheckButton")
		{
			FXCheckButton *checkButton=dynamic_cast<FXCheckButton*>(iter->getFoxWidget());
			if(checkButton->getCheck())
				(*w->result)[iter->getName()]="true";
			else
				(*w->result)[iter->getName()]="false";
		}
		else if(type=="ComboBox")
		{
			FXComboBox*listBox=dynamic_cast<FXComboBox*>(iter->getFoxWidget());
			//if(listBox->getCurrentItem()!=-1)
			//(*w->result)[iter->getName()]=listBox->getItem(listBox->getCurrentItem()).text();
			//else
			(*w->result)[iter->getName()]=listBox->getText().text();
		}
		
	}

	
	
	(*w->m_resultReady)=true;
	(*w->m_resultReadyCondition).notify_one();
	close();
	return 1;
}

long FoxInputDialog::onPressedEnter(FXObject *obj, FXSelector sel, void *ptr)
{

	if(defaultButton!="")
	{
		if(widgetsMap[defaultButton]!=NULL)
		onCmdClicked(widgetsMap[defaultButton],ID_CLICKED_BUTTON,NULL);
	}

	
}



