/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FoxMainWindow.hpp"
#include "TestPanel.hpp"
#include "FoxFileListPanel.hpp"
#include "FoxDockContainer.hpp"
#include "Exceptions.hpp"
#include <signal.h>
#include "config.h"

#include <boost/lexical_cast.hpp>
using boost::lexical_cast;

using namespace FX;
FXDEFMAP ( FoxMainWindow ) FoxMainWindowMap[] =
    {
        //________Message_Type_____________________ID____________Message_Handler_______
        FXMAPFUNCS( SEL_COMMAND, ID_QUIT,ID_RIGHT, FoxMainWindow::onMoveFrame ),
        FXMAPFUNC( SEL_COMMAND, FoxMainWindow::ID_CONFIGURE, FoxMainWindow::onOpenConfigurator),
        FXMAPFUNC( SEL_COMMAND, FoxMainWindow::ID_ABOUT, FoxMainWindow::onAbout),
        FXMAPFUNC( SEL_CONFIGURE, 0, FoxMainWindow::onConfigure),
        FXMAPFUNC( SEL_UPDATE, 0, FoxMainWindow::onUpdate),
        FXMAPFUNC(SEL_SIGNAL,FoxMainWindow::ID_CLOSEALL,FoxMainWindow::onCmdCloseAll),
        FXMAPFUNC(SEL_CLOSE,0,FoxMainWindow::onCmdCloseAll),
        
    };

FXIMPLEMENT ( FoxMainWindow, FXMainWindow, FoxMainWindowMap, ARRAYNUMBER ( FoxMainWindowMap ) )


FoxMainWindow::FoxMainWindow( FXApp * a )
 : FXMainWindow ( a, "nao", FoxSingleton<FoxIconManager>::instance(a).get("folder_big"), FoxSingleton<FoxIconManager>::instance(a).get("folder_small"), DECOR_ALL | LAYOUT_FIX_WIDTH, 0, 0, 600, 400, 0, 0, 0, 0, 0, 0),
   settings(singleton<Settings>::instance()),
   foxIconManager(FoxSingleton<FoxIconManager>::instance(a))    
{

	#ifndef DEBUG
	getApp()->addSignal(SIGINT,this,ID_CLOSEALL);
		#ifndef WIN32
		getApp()->addSignal(SIGQUIT,this,ID_CLOSEALL);
		getApp()->addSignal(SIGHUP,this,ID_CLOSEALL);
		getApp()->addSignal(SIGPIPE,this,ID_CLOSEALL);
		#endif
	#endif
	
	foxConfigurator=NULL;
	//FoxIconManager& foxIconManager=FoxSingleton<FoxIconManager>::instance(a);
		
	settings.addSettingsObservator( this );	
	
	int w=settings.getWindowWidth();
	int h=settings.getWindowHeight();
	resize(w,h);

	topdock = new FXDockSite ( this, LAYOUT_SIDE_TOP | LAYOUT_FILL_X );
	bottomdock = new FXDockSite ( this, LAYOUT_SIDE_BOTTOM | LAYOUT_FILL_X );
	leftdock = new FXDockSite ( this, LAYOUT_SIDE_LEFT | LAYOUT_FILL_Y );
	rightdock = new FXDockSite ( this, LAYOUT_SIDE_RIGHT | LAYOUT_FILL_Y );
            
        
            
	dock=new FoxDockContainer(topdock,bottomdock,leftdock,rightdock);
	_target=this;
		
	dragshell = new FXToolBarShell ( this, FRAME_RAISED );
	toolbar = new FXToolBar ( topdock, dragshell, LAYOUT_DOCK_SAME | LAYOUT_SIDE_TOP | FRAME_RAISED );
	new FXToolBarGrip ( toolbar, toolbar, FXToolBar::ID_TOOLBARGRIP, TOOLBARGRIP_SINGLE );
	//toolbar->setDockingSide(LAYOUT_SIDE_RIGHT);
	
	mainFrame = new FXVerticalFrame ( this, LAYOUT_FILL_X | LAYOUT_FILL_Y| FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	controlframe = new FXVerticalFrame ( mainFrame, LAYOUT_FILL_X | FRAME_NONE, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0 );
	splitter = new FXSplitter ( mainFrame, LAYOUT_FILL_X | SPLITTER_TRACKING | LAYOUT_FILL_Y );
	leftSide = new FXVerticalFrame ( splitter, LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_NONE, 0, 0, w / 2, 0, 5, 0, 0, 0 );
	rightSide = new FXVerticalFrame ( splitter, LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_NONE, 0, 0, w / 2, 0, 0, 5, 0, 0 );
		
			
	leftcontrolframe = new FXVerticalFrame ( leftSide, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0 );
	leftframe = new FXVerticalFrame ( leftSide, LAYOUT_FILL_X | LAYOUT_FILL_Y , 0, 0, 0, 0, 0, 0, 0, 0, 0 ,0 );
	rightcontrolframe = new FXVerticalFrame ( rightSide, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0 );
	rightframe = new FXVerticalFrame ( rightSide, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );

	new FXButton ( toolbar, "\tconfiguration", foxIconManager.get("configure"), this, FoxMainWindow::ID_CONFIGURE, FRAME_RAISED | LAYOUT_TOP | LAYOUT_LEFT | BUTTON_TOOLBAR, 0, 0, 0, 0, 0, 0, 0, 0);
        new FXButton ( toolbar, "\tabout", foxIconManager.get("about"), this, FoxMainWindow::ID_ABOUT,FRAME_RAISED | LAYOUT_TOP | LAYOUT_LEFT | BUTTON_TOOLBAR, 0, 0, 0, 0, 0, 0, 0, 0);
           
           /*
	FoxPanel*f=new FoxFileListPanel(this,"");
	FoxPanel*f2=new TestPanel(this,"");
	panels.push_back(f);
	panels.push_back(f2);
	*/
	
	leftPanel=new FoxFileListPanel(this,"left");
	panels.push_back(leftPanel);
//	leftPanel->frame->setBackColor(FXRGB(0,255,0));
	leftPanel->moveToLeft();
	
	
	rightPanel=new FoxFileListPanel(this,"right");
	panels.push_back(rightPanel);
//	rightPanel->frame->setBackColor(FXRGB(255,0,0));
	rightPanel->moveToRight();
	

	leftPanel->setOpposite( rightPanel ); 
	rightPanel->setOpposite( leftPanel ); 
	 
	 ratio=0.5;
	doublePanel=true; 
	if(settings.isSinglePanel())
	{
		ratio=0;
		doublePanel=false;
		setPanelWidth(leftPanel,0);
	} 
	
	rightPanel->setFocus();
}


FoxMainWindow::~FoxMainWindow()
{

	delete dock;
	delete dragshell;
}



void FoxMainWindow::create ()
{
    FXMainWindow::create ();
    dragshell->create();
    
    string placement=settings.getStartupPlacement();
    if(placement=="center")
    show ( PLACEMENT_SCREEN );
    else    
    show ( PLACEMENT_CURSOR );
}

long FoxMainWindow::onMoveFrame( FXObject * sender, FXSelector sel , void *ptr )
{

	FoxPanel *panel;

	if(sender!=NULL)
	{
	FXButton * bt = ( FXButton * ) sender;
	panel = ( FoxPanel * ) bt->getUserData ();
	}
	else
	panel=static_cast<FoxPanel*>(ptr);


    FXushort id = FXSELID ( sel );
    
    if(!doublePanel && id!=ID_QUIT)
    {
    	if(rightSide->getWidth()==0)
    	id=ID_LEFT;
    	else
    	id=ID_RIGHT;
    }
    
    if ( id == ID_RIGHT )
    {
    	
    	rightPanel->moveToTop();   
    	panel->moveToRight();
    	rightPanel=panel;
    	//rightframe->setFocus();
    }
    else if ( id == ID_LEFT )
    {
    	
    	leftPanel->moveToTop();    	
    	panel->moveToLeft();    	
    	leftPanel=panel;
    	//leftframe->setFocus();
    }
    else if ( id == ID_QUIT )
    {
    	if(panel->canClose())
    	{
    		vector<Panel*>::iterator iter;
    		iter=find(panels.begin(),panels.end(),panel);
    		if(iter!=panels.end())
    		{
    			panels.erase(iter);
    		}
    		delete panel;
    		panel=NULL;
    	}
    	else
    	{
    		/*TODO*/

    	}
    	
    	return 1;
    }
	

	leftPanel->setOpposite( rightPanel ); 
	rightPanel->setOpposite( leftPanel );
	
	
	FoxPanel *foxPanel=dynamic_cast<FoxPanel*>(panel);	
	foxPanel->setFocus();
	
	return 1;
    
}

FoxPanel * FoxMainWindow::getOppositePanel( FoxPanel *panel )
{
	if(panel==leftPanel)
		return rightPanel;
	else if(panel==rightPanel)
		return leftPanel;
	else
		return NULL;
}

long FoxMainWindow::onOpenConfigurator( FXObject * sender, FXSelector , void * )
{
	/*if(foxConfigurator)
	{
	delete foxConfigurator;	
	}*/
	foxConfigurator=new FoxConfigurator(this);
	foxConfigurator->create();
	foxConfigurator->show ( PLACEMENT_OWNER );

}

void FoxMainWindow::notify( void )
{	
	resize(settings.getWindowWidth(),settings.getWindowHeight());
}

long FoxMainWindow::onConfigure( FXObject * sender, FXSelector sel , void *ptr )
{

	FXMainWindow::onConfigure ( sender, sel, ptr );
	if(doublePanel)
    	{
    	float width = this->getWidth () * ratio;
    	leftSide->setWidth ( static_cast<int>( width ) );
    	
    	}
    	else
    	{
    		if(leftSide->getWidth() > rightSide->getWidth())
    		//leftSide->setWidth(getWidth()-margin);
    		rightSide->setWidth(0);
    		else
    		leftSide->setWidth(0);
    	}
    	update();
    	
  
    	
    	
}

long FoxMainWindow::onUpdate( FXObject * sender, FXSelector sel , void *ptr )
{

	FXMainWindow::onUpdate ( sender, sel, ptr );    	
		float l = leftSide->getWidth ();
		float w = getWidth ();
		ratio = ( float ) l / ( float ) w;

}



void FoxMainWindow::setPanelWidth( FoxPanel * panel, int width )
{

	if(width==-1)
	{
		leftSide->setWidth(getWidth()/2);
		doublePanel=true;
	return;
	}
	
	
	if(panel==leftPanel)
	{
		leftSide->setWidth(width);
		margin=mainFrame->getWidth()-leftSide->getWidth();
	}
	else if(panel==rightPanel)
	{
		leftSide->setWidth(mainFrame->getWidth()-width);
	}
	
	doublePanel=false;
}

long FoxMainWindow::onCmdCloseAll(FXObject * sender, FXSelector , void *)
{

	#ifdef DEBUG
	cout<<"onCmdCloseAll"<<endl;
	#endif
	

	
	/*TODO*/

	vector<Panel*>::iterator iter;
	bool closeAllowed=true;
	for(iter=panels.begin();iter!=panels.end();++iter)
	{
		if(!(*iter)->canClose())
		{
		closeAllowed=false;
		break;
		}
		
	}
	
	if(closeAllowed)
	{
		for(iter=panels.begin();iter!=panels.end();++iter)
		delete *iter;
		
		settings.removeSettingsObservator( this );
		
		if(settings.getSaveWindowSize())
		{
			settings.setWindowHeight(getHeight());
			settings.setWindowWidth(getWidth());
		}
		
			
		
		
		
	close(true);
	}
}

void FoxMainWindow::createPanel(string name, string param)
{
	
	if(name=="FileList")
	{
		try
		{
			FoxFileListPanel *panel=new FoxFileListPanel(this,param);
			panel->create();
			panels.push_back(panel);
			
			int id=ID_RIGHT;			
			if(leftframe->hasFocus())
			id=ID_LEFT;
					
 			this->handle(NULL,FXSEL(SEL_COMMAND,id),panel);
			
		}
		catch(ExceptionSupportNotCompiled &e)
		{
			FXMessageBox::error (getApp(), MBOX_OK, "error", "Support for this feature not compiled. See http://nao.linux.pl how to compile it.");
		}
	}
	else /*TODO*/
	{
	
	}
	
}

long FoxMainWindow::onAbout(FXObject * sender, FXSelector , void *)
{
	string msg = "Nao file manager " + string( VERSION ) + "\n\nby Mateusz Dworak (http://nao.linux.pl)\n\npublished under the GNU General Public License\nUsing the FOX C++ GUI Library (http://www.fox-tookit.org) \n\nDefault icon set: Gnome-mix\nLogo: Patrycjusz Szubryt (http://3ga.pl)";

	//FXMessageBox::information (getApp(), MBOX_OK, "about", msg.c_str());
	FXMessageBox (getApp(), "about", msg.c_str(), foxIconManager.get("logo"),MBOX_OK).execute();
		
}
