/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef FOXMAINWINDOW_H
#define FOXMAINWINDOW_H

#include <fx.h>
#include "FoxPanel.hpp"
#include "Settings.hpp"
#include "FoxIconManager.hpp"
#include "FoxPanelManager.hpp"
#include "FoxDockContainer.hpp"
#include "FoxConfigurator.hpp"
#include "SettingsObservator.hpp"




/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class FoxMainWindow : public FXMainWindow, public FoxPanelManager, public SettingsObservator
{

    FXDECLARE (FoxMainWindow)
    
public:  
  
    enum
    {
	ID_NEWFRAME = FXMainWindow::ID_LAST,
	ID_CONFIGURE,
	ID_CLOSEALL,
	ID_ABOUT,
    };


    FoxMainWindow(FXApp *);
    virtual ~FoxMainWindow();    
    virtual void create ();
    long onMoveFrame (FXObject * sender, FXSelector, void *);
    long onOpenConfigurator (FXObject * sender, FXSelector, void *);
    long onConfigure (FXObject * sender, FXSelector, void *);
    long onUpdate (FXObject * sender, FXSelector, void *);
    long onCmdCloseAll (FXObject * sender, FXSelector, void *);
    long onAbout (FXObject * sender, FXSelector, void *);
    
    FoxPanel *getOppositePanel(FoxPanel *panel);
    void notify(void);
    void setPanelWidth(FoxPanel *panel,int width);
    void createPanel(string name, string param );
    
    
    	FXComposite* getControlframe() const
	{
	    return controlframe;
	}
	
	FXComposite* getLeftcontrolframe() const
	{
	    return leftcontrolframe;
	}
	
	FXComposite* getRightframe() const
	{
	    return rightframe;
	}
	
	FXComposite* getRightcontrolframe() const
	{
	    return rightcontrolframe;
	}
	
	FXComposite* getLeftframe() const
	{
	    return leftframe;
	}

	FXObject* getTarget() const
	{
	    return _target;
	}

	FoxDockContainer* getDock() const
	{
	    return dock;
	}
    
        
private:
     FoxMainWindow ():settings(singleton<Settings>::instance()),foxIconManager(FoxSingleton<FoxIconManager>::instance(NULL)){}
    
	FXVerticalFrame *controlframe,*leftSide,*rightSide,*leftcontrolframe,*leftframe,*rightcontrolframe,*rightframe,*mainFrame; 
	FXSplitter *splitter;
	FoxPanel *rightPanel,*leftPanel;
	Settings& settings;
	
	FXDockSite *topdock,*bottomdock,*rightdock,*leftdock;
	FXToolBar *toolbar;
	FXToolBarShell *dragshell;
	FoxDockContainer *dock;
	FXObject *_target;
	FoxConfigurator *foxConfigurator;
	FoxIconManager& foxIconManager;
	float ratio;
	bool doublePanel;
	int margin;
	vector<Panel*>panels;

};

#endif
