/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "VfsFlatDirectory.hpp"
#include "FilesEngine.hpp"

REGISTER_FACTORABLE(VfsFlatDirectory);

class OperationGetFlatDirectory
{
public:	
	
	OperationGetFlatDirectory(vector<File> &_files):files(_files)
	{
	}
	
	void process(File file)
	{
		if(!file.isDirectory())
		{
			files.push_back(file);
		}
		else
		{
		}
	}

private:
	vector<File>& files;
	
};



VfsFlatDirectory::VfsFlatDirectory()
{
	
}


VfsFlatDirectory::~VfsFlatDirectory()
{
}

Directory VfsFlatDirectory::openDir(path & p, Cancelable *cancelable)
{
	files.clear();
	path pp=path(params["directory"]);
	
	
	OperationGetFlatDirectory operation(files);
	File file=readFile(pp);
	VfsLocal vfsLocal;
	try
	{
	vfsLocal.goRecursive(file,operation,false,cancelable);  
	}
	catch(...){}
	iter=files.begin(); 
	Directory directory(this,NULL,pp);
	return directory;
	

}

void VfsFlatDirectory::init(Panel * _panel, QuestionManager * _questionManager)
{
	VfsBase::init(_panel,_questionManager);
	params=FilesEngine::parseParam(panel->getParam());	
	
	rootDir=params["directory"];
	
	panel->setLabel(params["directory"],"flat");
	
	vector<string>headers=settings.getHeaders("FlatDirectory");
	
	//setHeaders(settings.getHeaders("FlatDirectory"));
	
	vector<string>::iterator iter;
	for(iter=headers.begin();iter!=headers.end();++iter)
	{
		if(*iter=="size")
		fileInfoVector.push_back(&VfsBase::setFileSize);
		else if(*iter=="owner")
		fileInfoVector.push_back(&VfsBase::setFileOwner);
		else if(*iter=="group")
		fileInfoVector.push_back(&VfsBase::setFileGroup);
		else if(*iter=="mime")
		fileInfoVector.push_back(&VfsBase::setFileMime);
		else if(*iter=="mode")
		fileInfoVector.push_back(&VfsBase::setFilePermissions);	
		else if(*iter=="atime")
		fileInfoVector.push_back(&VfsBase::setAccessTime);	
		else if(*iter=="mtime")
		fileInfoVector.push_back(&VfsBase::setModificationTime);
		else if(*iter=="ctime")
		fileInfoVector.push_back(&VfsBase::setCreationTime);	
		else if(*iter=="path")
		fileInfoVector.push_back(&VfsBase::setPath);	
		else if(*iter=="relativepath")
		fileInfoVector.push_back(&VfsFlatDirectory::setRelativePath);
	}
	
	
}

void VfsFlatDirectory::setRelativePath(vector< string > & fields, File & file)
{
	string parentDir;
	
	if(rootDir.size()!=file.getParentDirectory().string().size())
	{	parentDir=file.getParentDirectory().string();
		parentDir=parentDir.substr(rootDir.size()+1,parentDir.size()-rootDir.size()-1);	
	}
	
	fields.push_back(parentDir);
}



File VfsFlatDirectory::readDir(Directory * directory)
{

	if(iter!=files.end())
	return *iter++;
	else
	throw ExceptionReadDirEnd();
	
}

File VfsFlatDirectory::readFullFile(File file)
{
	if(!file.isDirectory())
	file.setMime(mime.getFileMime(file.getPath(),VfsTypeLocal));
	else
	file.setMime("special/directory");
	
	vector<string>fields;	

	for(fileInfoVectorIter=fileInfoVector.begin();fileInfoVectorIter!=fileInfoVector.end();++fileInfoVectorIter)
	{
		(this->*(*fileInfoVectorIter))(fields,file);
	}
	

	file.setFields(fields);
	return file;
}



